@@ -56,6 +56,10 @@ enum proc_gen proc_gen;
 static uint64_t kernel_entry;
 static bool kernel_32bit;
 
+#ifdef SKIBOOT_GCOV
+void skiboot_gcov_done(void);
+#endif
+
 struct debug_descriptor debug_descriptor = {
 	.eye_catcher	= "OPALdbug",
 	.version	= DEBUG_DESC_VERSION,
@@ -284,47 +288,72 @@ extern char __builtin_kernel_start[];
 extern char __builtin_kernel_end[];
 extern uint64_t boot_offset;
 
-static bool load_kernel(void)
+static size_t kernel_size;
+static size_t initramfs_size;
+
+static bool start_preload_kernel(void)
 {
-	struct elf_hdr *kh;
-	size_t ksize;
 	int loaded;
 
-	prlog(PR_NOTICE, "INIT: Loading kernel\n");
-
 	/* Try to load an external kernel payload through the platform hooks */
-	ksize = KERNEL_LOAD_SIZE;
+	kernel_size = KERNEL_LOAD_SIZE;
 	loaded = start_preload_resource(RESOURCE_ID_KERNEL,
 					RESOURCE_SUBID_NONE,
 					KERNEL_LOAD_BASE,
-					&ksize);
-	if (loaded == OPAL_SUCCESS)
-		loaded = wait_for_resource_loaded(RESOURCE_ID_KERNEL,
-						  RESOURCE_SUBID_NONE);
+					&kernel_size);
+	if (loaded != OPAL_SUCCESS) {
+		printf("INIT: platform start load kernel failed\n");
+		kernel_size = 0;
+		return false;
+	}
+
+	initramfs_size = INITRAMFS_LOAD_SIZE;
+	loaded = start_preload_resource(RESOURCE_ID_INITRAMFS,
+					RESOURCE_SUBID_NONE,
+					INITRAMFS_LOAD_BASE, &initramfs_size);
+	if (loaded != OPAL_SUCCESS) {
+		printf("INIT: platform start load initramfs failed\n");
+		initramfs_size = 0;
+		return false;
+	}
+
+	return true;
+}
+
+static bool load_kernel(void)
+{
+	struct elf_hdr *kh;
+	int loaded;
+
+	prlog(PR_NOTICE, "INIT: Waiting for kernel...\n");
+
+	loaded = wait_for_resource_loaded(RESOURCE_ID_KERNEL,
+					  RESOURCE_SUBID_NONE);
 
 	if (loaded != OPAL_SUCCESS) {
-		printf("INIT: platform kernel load failed\n");
-		ksize = 0;
+		printf("INIT: platform wait for kernel load failed\n");
+		kernel_size = 0;
 	}
 
 	/* Try embedded kernel payload */
-	if (!ksize) {
-		ksize = __builtin_kernel_end - __builtin_kernel_start;
-		if (ksize) {
+	if (!kernel_size) {
+		kernel_size = __builtin_kernel_end - __builtin_kernel_start;
+		if (kernel_size) {
 			/* Move the built-in kernel up */
 			uint64_t builtin_base =
 				((uint64_t)__builtin_kernel_start) -
 				SKIBOOT_BASE + boot_offset;
 			printf("Using built-in kernel\n");
-			memmove(KERNEL_LOAD_BASE, (void*)builtin_base, ksize);
+			memmove(KERNEL_LOAD_BASE, (void*)builtin_base,
+				kernel_size);
 		}
 	}
 
-	if (!ksize)
+	if (!kernel_size)
 		printf("Assuming kernel at %p\n", KERNEL_LOAD_BASE);
 
 	printf("INIT: Kernel loaded, size: %zu bytes (0 = unknown preload)\n",
-	       ksize);
+	       kernel_size);
 
 	kh = (struct elf_hdr *)KERNEL_LOAD_BASE;
 	if (kh->ei_class == ELF_CLASS_64)
@@ -338,29 +367,24 @@ static bool load_kernel(void)
 
 static void load_initramfs(void)
 {
-	size_t size;
 	int loaded;
 
-	size = INITRAMFS_LOAD_SIZE;
-	loaded = start_preload_resource(RESOURCE_ID_INITRAMFS,
-					RESOURCE_SUBID_NONE,
-					INITRAMFS_LOAD_BASE, &size);
+	loaded = wait_for_resource_loaded(RESOURCE_ID_INITRAMFS,
+					  RESOURCE_SUBID_NONE);
 
-	if (loaded == OPAL_SUCCESS)
-		loaded = wait_for_resource_loaded(RESOURCE_ID_INITRAMFS,
-						  RESOURCE_SUBID_NONE);
-
-	if (loaded != OPAL_SUCCESS || !size)
+	if (loaded != OPAL_SUCCESS || !initramfs_size)
 		return;
 
-	printf("INIT: Initramfs loaded, size: %zu bytes\n", size);
+	printf("INIT: Initramfs loaded, size: %zu bytes\n", initramfs_size);
 
 	dt_add_property_u64(dt_chosen, "linux,initrd-start",
 			(uint64_t)INITRAMFS_LOAD_BASE);
 	dt_add_property_u64(dt_chosen, "linux,initrd-end",
-			(uint64_t)INITRAMFS_LOAD_BASE + size);
+			(uint64_t)INITRAMFS_LOAD_BASE + initramfs_size);
 }
 
+int64_t mem_dump_free(void);
+
 void __noreturn load_and_boot_kernel(bool is_reboot)
 {
 	const struct dt_property *memprop;
@@ -431,6 +455,8 @@ void __noreturn load_and_boot_kernel(bool is_reboot)
 
 	cpu_give_self_os();
 
+	mem_dump_free();
+
 	printf("INIT: Starting kernel at 0x%llx, fdt at %p (size 0x%x)\n",
 	       kernel_entry, fdt, fdt_totalsize(fdt));
 
@@ -516,6 +542,17 @@ static void setup_branch_null_catcher(void)
 /* Called from head.S, thus no prototype. */
 void __noreturn main_cpu_entry(const void *fdt, u32 master_cpu);
 
+typedef void (*ctorcall_t)(void);
+
+static void do_ctors(void)
+{
+	extern ctorcall_t __ctors_start[], __ctors_end[];
+	ctorcall_t *call;
+
+	for (call = __ctors_start; call < __ctors_end; call++)
+		(*call)();
+}
+
 void __noreturn main_cpu_entry(const void *fdt, u32 master_cpu)
 {
 	/*
@@ -544,12 +581,18 @@ void __noreturn main_cpu_entry(const void *fdt, u32 master_cpu)
 	 */
 	setup_branch_null_catcher();
 
+	do_ctors();
+
 	printf("SkiBoot %s starting...\n", version);
 	printf("initial console log level: memory %d, driver %d\n",
 	       (debug_descriptor.console_log_levels >> 4),
 	       (debug_descriptor.console_log_levels & 0x0f));
 	prlog(PR_TRACE, "You will not see this\n");
 
+#ifdef SKIBOOT_GCOV
+	skiboot_gcov_done();
+#endif
+
 	/* Initialize boot cpu's cpu_thread struct */
 	init_boot_cpu();
 
@@ -683,6 +726,10 @@ void __noreturn main_cpu_entry(const void *fdt, u32 master_cpu)
 	/* Read in NVRAM and set it up */
 	nvram_init();
 
+	phb3_preload_vpd();
+	phb3_preload_capp_ucode();
+	start_preload_kernel();
+
 	/* NX init */
 	nx_init();
 
@@ -744,8 +791,15 @@ void __noreturn __secondary_cpu_entry(void)
 
 		/* Relax a bit to give the simulator some breathing space */
 		i = 1000;
-		while (--i)
-			cpu_relax();
+		smt_very_low();
+		asm volatile("mtctr %0;\n"
+			     "1: nop; nop; nop; nop;\n"
+			     "   nop; nop; nop; nop;\n"
+			     "   nop; nop; nop; nop;\n"
+			     "   nop; nop; nop; nop;\n"
+			     "   bdnz 1b"
+			     : : "r" (i) : "memory", "ctr");
+		smt_medium();
 	}
 }
 
