@@ -25,6 +25,9 @@
 #include <mem_region.h>
 #include <mem_region-malloc.h>
 
+int64_t mem_dump_free(void);
+void mem_dump_allocs(void);
+
 /* Memory poisoning on free (if POISON_MEM_REGION set to 1) */
 #define POISON_MEM_REGION	0
 #define POISON_MEM_REGION_WITH	0x99
@@ -258,7 +261,7 @@ static bool region_is_reserved(struct mem_region *region)
 	return region->type != REGION_OS;
 }
 
-static void mem_dump_allocs(void)
+void mem_dump_allocs(void)
 {
 	struct mem_region *region;
 	struct alloc_hdr *hdr;
@@ -285,6 +288,39 @@ static void mem_dump_allocs(void)
 	}
 }
 
+int64_t mem_dump_free(void)
+{
+	struct mem_region *region;
+	struct alloc_hdr *hdr;
+	int64_t total_free;
+	int64_t region_free;
+
+	total_free = 0;
+
+	printf("Free space in HEAP memory regions:\n");
+	list_for_each(&regions, region, list) {
+		if (region->type != REGION_SKIBOOT_HEAP)
+			continue;
+		region_free = 0;
+
+		if (region->free_list.n.next == NULL) {
+			continue;
+		}
+		for (hdr = region_start(region); hdr; hdr = next_hdr(region, hdr)) {
+			if (!hdr->free)
+				continue;
+
+			region_free+= hdr->num_longs * sizeof(long);
+		}
+		printf("Region %s free: %llu\n", region->name, region_free);
+		total_free += region_free;
+	}
+
+	printf("Total free: %llu\n", total_free);
+
+	return total_free;
+}
+
 static void *__mem_alloc(struct mem_region *region, size_t size, size_t align,
 			 const char *location)
 {
@@ -753,12 +789,135 @@ struct mem_region *find_mem_region(const char *name)
 	return NULL;
 }
 
-/* Trawl through device tree, create memory regions from nodes. */
-void mem_region_init(void)
+bool mem_range_is_reserved(uint64_t start, uint64_t size)
+{
+	uint64_t end = start + size;
+	struct mem_region *region;
+
+	/* We may have the range covered by a number of regions, which could
+	 * appear in any order. So, we look for a region that covers the
+	 * start address, and bump start up to the end of that region.
+	 *
+	 * We repeat until we've either bumped past the end of the range,
+	 * or we didn't find a matching region.
+	 *
+	 * This has a worst-case of O(n^2), but n is well bounded by the
+	 * small number of reservations.
+	 */
+	for (;;) {
+		bool found = false;
+
+		list_for_each(&regions, region, list) {
+			if (!region_is_reserved(region))
+				continue;
+
+			/* does this region overlap the start address, and
+			 * have a non-zero size? */
+			if (region->start <= start &&
+					region->start + region->len > start &&
+					region->len) {
+				start = region->start + region->len;
+				found = true;
+			}
+		}
+
+		/* 'end' is the first byte outside of the range */
+		if (start >= end)
+			return true;
+
+		if (!found)
+			break;
+	}
+
+	return false;
+}
+
+void adjust_cpu_stacks_alloc(void)
+{
+	/* CPU stacks start at 0, then when we know max possible PIR,
+	 * we adjust, then when we bring all CPUs online we know the
+	 * runtime max PIR, so we adjust this a few times during boot.
+	 */
+	skiboot_cpu_stacks.len = (cpu_max_pir + 1) * STACK_SIZE;
+}
+
+static void mem_region_parse_reserved_properties(void)
 {
 	const struct dt_property *names, *ranges;
 	struct mem_region *region;
+
+	prlog(PR_INFO, "MEM: parsing reserved memory from "
+			"reserved-names/-ranges properties\n");
+
+	names = dt_find_property(dt_root, "reserved-names");
+	ranges = dt_find_property(dt_root, "reserved-ranges");
+	if (names && ranges) {
+		const uint64_t *range;
+		int n, len;
+
+		range = (const void *)ranges->prop;
+
+		for (n = 0; n < names->len; n += len, range += 2) {
+			char *name;
+
+			len = strlen(names->prop + n) + 1;
+			name = strdup(names->prop + n);
+
+			region = new_region(name,
+					dt_get_number(range, 2),
+					dt_get_number(range + 1, 2),
+					NULL, REGION_HW_RESERVED);
+			list_add(&regions, &region->list);
+		}
+	} else if (names || ranges) {
+		prerror("Invalid properties: reserved-names=%p "
+				"with reserved-ranges=%p\n",
+				names, ranges);
+		abort();
+	} else {
+		return;
+	}
+}
+
+static bool mem_region_parse_reserved_nodes(const char *path)
+{
+	struct dt_node *parent, *node;
+
+	parent = dt_find_by_path(dt_root, path);
+	if (!parent)
+		return false;
+
+	prlog(PR_INFO, "MEM: parsing reserved memory from node %s\n", path);
+
+	dt_for_each_child(parent, node) {
+		const struct dt_property *reg;
+		struct mem_region *region;
+
+		reg = dt_find_property(node, "reg");
+		if (!reg) {
+			char *nodepath = dt_get_path(node);
+			prerror("node %s has no reg property, ignoring\n",
+					nodepath);
+			free(nodepath);
+			continue;
+		}
+
+		region = new_region(strdup(node->name),
+				dt_get_number(reg->prop, 2),
+				dt_get_number(reg->prop + sizeof(u64), 2),
+				node, REGION_HW_RESERVED);
+		list_add(&regions, &region->list);
+	}
+
+	return true;
+}
+
+/* Trawl through device tree, create memory regions from nodes. */
+void mem_region_init(void)
+{
+	struct mem_region *region;
 	struct dt_node *i;
+	bool rc;
 
 	/* Ensure we have no collision between skiboot core and our heap */
 	extern char _end[];
@@ -801,8 +960,7 @@ void mem_region_init(void)
 		unlock(&mem_region_lock);
 	}
 
-	/* Now we know how many CPU stacks we have, fix that up. */
-	skiboot_cpu_stacks.len = (cpu_max_pir + 1) * STACK_SIZE;
+	adjust_cpu_stacks_alloc();
 
 	lock(&mem_region_lock);
 
@@ -817,42 +975,15 @@ void mem_region_init(void)
 	}
 
 	/* Add reserved ranges from the DT */
-	names = dt_find_property(dt_root, "reserved-names");
-	ranges = dt_find_property(dt_root, "reserved-ranges");
-	if (names && ranges) {
-		const uint64_t *range;
-		int n, len;
-
-		range = (const void *)ranges->prop;
-
-		for (n = 0; n < names->len; n += len, range += 2) {
-			char *name;
-
-			len = strlen(names->prop + n) + 1;
-			name = strdup(names->prop + n);
-
-			region = new_region(name,
-					dt_get_number(range, 2),
-					dt_get_number(range + 1, 2),
-					NULL, REGION_HW_RESERVED);
-			list_add(&regions, &region->list);
-		}
-	} else if (names || ranges) {
-		prerror("Invalid properties: reserved-names=%p "
-				"with reserved-ranges=%p\n",
-				names, ranges);
-		abort();
-	}
+	rc = mem_region_parse_reserved_nodes("/reserved-memory");
+	if (!rc)
+		rc = mem_region_parse_reserved_nodes(
+				"/ibm,hostboot/reserved-memory");
+	if (!rc)
+		mem_region_parse_reserved_properties();
 
 	unlock(&mem_region_lock);
 
-	/* We generate the reservation properties from our own region list,
-	 * which now includes the existing data.
-	 */
-	if (names)
-		dt_del_property(dt_root, (struct dt_property *)names);
-	if (ranges)
-		dt_del_property(dt_root, (struct dt_property *)ranges);
 }
 
 static uint64_t allocated_length(const struct mem_region *r)
@@ -932,6 +1063,20 @@ static void mem_region_add_dt_reserved_node(struct dt_node *parent,
 {
 	char *name, *p;
 
+	/* If a reserved region was established before skiboot, it may be
+	 * referenced by a device-tree node with extra data. In that case,
+	 * copy the node to /reserved-memory/, unless it's already there.
+	 *
+	 * We update region->node to the new copy here, as the prd code may
+	 * update regions' device-tree nodes, and we want those updates to
+	 * apply to the nodes in /reserved-memory/.
+	 */
+	if (region->type == REGION_HW_RESERVED && region->node) {
+		if (region->node->parent != parent)
+			region->node = dt_copy(region->node, parent);
+		return;
+	}
+
 	name = strdup(region->name);
 
 	/* remove any cell addresses in the region name; we have our own cell
@@ -948,6 +1093,7 @@ static void mem_region_add_dt_reserved_node(struct dt_node *parent,
 void mem_region_add_dt_reserved(void)
 {
 	int names_len, ranges_len, len;
+	const struct dt_property *prop;
 	struct mem_region *region;
 	void *names, *ranges;
 	struct dt_node *node;
@@ -964,10 +1110,13 @@ void mem_region_add_dt_reserved(void)
 	mem_regions_finalised = true;
 
 	/* establish top-level reservation node */
-	node = dt_new(dt_root, "reserved-memory");
-	dt_add_property_cells(node, "#address-cells", 2);
-	dt_add_property_cells(node, "#size-cells", 2);
-	dt_add_property(node, "ranges", NULL, 0);
+	node = dt_find_by_path(dt_root, "reserved-memory");
+	if (!node) {
+		node = dt_new(dt_root, "reserved-memory");
+		dt_add_property_cells(node, "#address-cells", 2);
+		dt_add_property_cells(node, "#size-cells", 2);
+		dt_add_property(node, "ranges", NULL, 0);
+	}
 
 	/* First pass: calculate length of property data */
 	list_for_each(&regions, region, list) {
@@ -1002,6 +1151,15 @@ void mem_region_add_dt_reserved(void)
 	}
 	unlock(&mem_region_lock);
 
+
+	prop = dt_find_property(dt_root, "reserved-names");
+	if (prop)
+		dt_del_property(dt_root, (struct dt_property *)prop);
+
+	prop = dt_find_property(dt_root, "reserved-ranges");
+	if (prop)
+		dt_del_property(dt_root, (struct dt_property *)prop);
+
 	dt_add_property(dt_root, "reserved-names", names, names_len);
 	dt_add_property(dt_root, "reserved-ranges", ranges, ranges_len);
 
