@@ -21,6 +21,8 @@
 #include <timebase.h>
 #include <device.h>
 
+/* The eeh event code will need updating if this is ever increased to
+ * support more than 64 phbs */
 static struct phb *phbs[64];
 
 #define PCITRACE(_p, _bdfn, fmt, a...) \
@@ -169,6 +171,7 @@ static struct pci_device *pci_scan_one(struct phb *phb, struct pci_device *paren
 	pd->is_multifunction = !!(htype & 0x80);
 	pd->is_bridge = (htype & 0x7f) != 0;
 	pd->scan_map = 0xffffffff; /* Default */
+	pd->primary_bus = (bdfn >> 8);
 
 	/* On the upstream port of PLX bridge 8724 (rev ba), PCI_STATUS
 	 * register doesn't have capability indicator though it support
@@ -226,7 +229,6 @@ static struct pci_device *pci_scan_one(struct phb *phb, struct pci_device *paren
 	 * This will help when walking down those bridges later on
 	 */
 	if (pd->is_bridge) {
-		pd->primary_bus = (bdfn >> 8);
 		pci_cfg_write8(phb, bdfn, PCI_CFG_PRIMARY_BUS, pd->primary_bus);
 		pci_cfg_write8(phb, bdfn, PCI_CFG_SECONDARY_BUS, 0);
 		pci_cfg_write8(phb, bdfn, PCI_CFG_SUBORDINATE_BUS, 0);
@@ -301,8 +303,11 @@ static bool pci_enable_bridge(struct phb *phb, struct pci_device *pd)
 	bctl &= ~PCI_CFG_BRCTL_MABORT_REPORT;
 	pci_cfg_write16(phb, pd->bdfn, PCI_CFG_BRCTL, bctl);
 
-	/* PCI-E bridge, check the slot state */
-	if (pd->dev_type == PCIE_TYPE_ROOT_PORT ||
+	/* PCI-E bridge, check the slot state. We don't do that on the
+	 * root complex as this is handled separately and not all our
+	 * RCs implement the standard register set.
+	 */
+	if ((pd->dev_type == PCIE_TYPE_ROOT_PORT && pd->primary_bus > 0) ||
 	    pd->dev_type == PCIE_TYPE_SWITCH_DNPORT) {
 		uint16_t slctl, slcap, slsta, lctl;
 
@@ -603,17 +608,18 @@ static int pci_configure_mps(struct phb *phb,
 			     struct pci_device *pd,
 			     void *userdata __unused)
 {
-	uint32_t ecap, mps = phb->mps;
+	uint32_t ecap, mps;
 	uint16_t val;
 
+	assert(phb);
+	assert(pd);
+
+	mps = phb->mps;
 	/* If the MPS isn't acceptable one, bail immediately */
 	if (mps < 128 || mps > 4096)
 		return 1;
 
-	if (!phb || !pd)
-		return 0;
-
-	/* PCIe deivce always has MPS capacity */
+	/* PCIe device always has MPS capacity */
 	if (pd->mps) {
 		ecap = pci_cap(pd, PCI_CFG_CAP_ID_EXP, false);
 		mps = ilog2(mps) - 7;
@@ -758,7 +764,7 @@ static void pci_scan_phb(void *data)
 	       has_link ? "+downsteam" : " only");
 	pci_scan(phb, 0, 0xff, &phb->devices, NULL, has_link);
 
-	/* Configre MPS (Max Payload Size) for PCIe domain */
+	/* Configure MPS (Max Payload Size) for PCIe domain */
 	pci_walk_dev(phb, pci_get_mps, &mps);
 	phb->mps = mps;
 	pci_walk_dev(phb, pci_configure_mps, NULL);
@@ -1081,6 +1087,10 @@ void pci_std_swizzle_irq_map(struct dt_node *np,
 	}
 	map_size = esize * edevcount * 4 * sizeof(uint32_t);
 	map = p = zalloc(map_size);
+	if (!map) {
+		prerror("Failed to allocate interrupt-map-mask !\n");
+		return;
+	}
 
 	for (dev = 0; dev < edevcount; dev++) {
 		for (irq = 0; irq < 4; irq++) {
@@ -1142,10 +1152,14 @@ static void pci_add_slot_properties(struct phb *phb, struct pci_slot_info *info,
 	dt_add_property_string(np, "ibm,slot-label", info->label);
 }
 
-static void pci_add_loc_code(struct dt_node *np)
+static void pci_add_loc_code(struct dt_node *np, struct pci_device *pd)
 {
 	struct dt_node *p = np->parent;
 	const char *blcode = NULL;
+	char *lcode;
+	uint32_t class_code;
+	uint8_t class, sub;
+	uint8_t pos, len;
 
 	/* Look for a parent with a slot-location-code */
 	while (p && !blcode) {
@@ -1154,7 +1168,31 @@ static void pci_add_loc_code(struct dt_node *np)
 	}
 	if (!blcode)
 		return;
-	dt_add_property_string(np, "ibm,loc-code", blcode);
+
+	/* ethernet devices get port codes */
+	class_code = dt_prop_get_u32(np, "class-code");
+	class = class_code >> 16;
+	sub = (class_code >> 8) & 0xff;
+	if (class == 0x02 && sub == 0x00) {
+		/* There's usually several spaces at the end of the property.
+		   Test for, but don't rely on, that being the case */
+		len = strlen(blcode);
+		for (pos = 0; pos < len; pos++)
+			if (blcode[pos] == ' ') break;
+		if (pos + 3 < len)
+			lcode = strdup(blcode);
+		else {
+			lcode = malloc(pos + 3);
+			memcpy(lcode, blcode, len);
+		}
+		lcode[pos++] = '-';
+		lcode[pos++] = 'T';
+		lcode[pos++] = (char)(pd->bdfn & 0x7) + '1';
+		lcode[pos++] = '\0';
+		dt_add_property_string(np, "ibm,loc-code", lcode);
+		free(lcode);
+	} else
+		dt_add_property_string(np, "ibm,loc-code", blcode);
 }
 
 static void pci_print_summary_line(struct phb *phb, struct pci_device *pd,
@@ -1285,7 +1323,7 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 		pci_add_slot_properties(phb, pd->slot_info, np);
 
 	/* Make up location code */
-	pci_add_loc_code(np);
+	pci_add_loc_code(np, pd);
 
 	/* XXX FIXME: We don't look for BARs, we only put the config space
 	 * entry in the "reg" property. That's enough for Linux and we might
@@ -1379,7 +1417,6 @@ void pci_reset(void)
 
 static void pci_do_jobs(void (*fn)(void *))
 {
-	struct cpu_thread *cpu = first_available_cpu();
 	void *jobs[ARRAY_SIZE(phbs)];
 	int i;
 
@@ -1389,36 +1426,15 @@ static void pci_do_jobs(void (*fn)(void *))
 			continue;
 		}
 
-		/* Don't queue job to current CPU, which is always
-		 * the poller.
-		 */
-		while (cpu) {
-			if (cpu != this_cpu())
-				break;
-
-			cpu = next_available_cpu(cpu);
-			if (!cpu)
-				cpu = first_available_cpu();
-
-			/* No CPU to run on, just run synchro */
-			if (cpu == this_cpu()) {
-				fn(phbs[i]);
-				jobs[i] = NULL;
-				goto next_phb;
-			}
-		}
-
-		jobs[i] = __cpu_queue_job(cpu, fn, phbs[i], false);
+		jobs[i] = __cpu_queue_job(NULL, phbs[i]->dt_node->name,
+					  fn, phbs[i], false);
 		assert(jobs[i]);
 
-		/* For next CPU */
-		cpu = next_available_cpu(cpu);
-		if (!cpu)
-			cpu = first_available_cpu();
-	next_phb:
-		;
 	}
 
+	/* If no secondary CPUs, do everything sync */
+	cpu_process_local_jobs();
+
 	/* Wait until all tasks are done */
 	for (i = 0; i < ARRAY_SIZE(phbs); i++) {
 		if (!jobs[i])
