@@ -60,6 +60,9 @@ struct prd_range {
 	const char		*name;
 	uint64_t		physaddr;
 	uint64_t		size;
+	void			*buf;
+	bool			multiple;
+	uint32_t		instance;
 };
 
 struct opal_prd_ctx {
@@ -68,6 +71,7 @@ struct opal_prd_ctx {
 	struct opal_prd_info	info;
 	struct prd_range	*ranges;
 	int			n_ranges;
+	bool			fw_range_instances;
 	long			page_size;
 	void			*code_addr;
 	size_t			code_size;
@@ -127,7 +131,7 @@ struct func_desc {
 	void *toc;
 } hbrt_entry;
 
-static struct prd_range *find_range(const char *name)
+static struct prd_range *find_range(const char *name, uint32_t instance)
 {
 	struct prd_range *range;
 	unsigned int i;
@@ -135,8 +139,13 @@ static struct prd_range *find_range(const char *name)
 	for (i = 0; i < ctx->n_ranges; i++) {
 		range = &ctx->ranges[i];
 
-		if (!strcmp(range->name, name))
-			return range;
+		if (strcmp(range->name, name))
+			continue;
+
+		if (range->multiple && range->instance != instance)
+			continue;
+
+		return range;
 	}
 
 	return NULL;
@@ -273,39 +282,52 @@ int hservice_scom_write(uint64_t chip_id, uint64_t addr,
 	return 0;
 }
 
-uint64_t hservice_get_reserved_mem(const char *name)
+uint64_t hservice_get_reserved_mem(const char *name, uint32_t instance)
 {
-	uint64_t align_physaddr, offset;
 	struct prd_range *range;
-	void *addr;
 
-	pr_debug("IMAGE: hservice_get_reserved_mem: %s", name);
+	pr_debug("IMAGE: hservice_get_reserved_mem: %s, %d", name, instance);
 
-	range = find_range(name);
+	range = find_range(name, instance);
 	if (!range) {
 		pr_log(LOG_WARNING, "IMAGE: get_reserved_mem: "
 				"no such range %s", name);
 		return 0;
 	}
 
-	pr_debug("IMAGE: Mapping 0x%016lx 0x%08lx %s",
-			range->physaddr, range->size, range->name);
+	if (!range->buf) {
+		uint64_t align_physaddr, offset;
 
-	align_physaddr = range->physaddr & ~(ctx->page_size-1);
-	offset = range->physaddr & (ctx->page_size-1);
-	addr = mmap(NULL, range->size, PROT_WRITE | PROT_READ,
-				MAP_SHARED, ctx->fd, align_physaddr);
+		pr_debug("IMAGE: Mapping 0x%016lx 0x%08lx %s[%d]",
+				range->physaddr, range->size,
+				range->name, range->instance);
 
-	if (addr == MAP_FAILED) {
-		pr_log(LOG_ERR, "IMAGE: mmap of %s(0x%016lx) failed: %m",
-				name, range->physaddr);
+		align_physaddr = range->physaddr & ~(ctx->page_size-1);
+		offset = range->physaddr & (ctx->page_size-1);
+		range->buf = mmap(NULL, range->size, PROT_WRITE | PROT_READ,
+					MAP_SHARED, ctx->fd, align_physaddr);
+
+		if (range->buf == MAP_FAILED)
+			pr_log(LOG_ERR,
+				"IMAGE: mmap of %s[%d](0x%016lx) failed: %m",
+				name, instance, range->physaddr);
+		else
+			range->buf += offset;
+	}
+
+	if (range->buf == MAP_FAILED) {
+		pr_log(LOG_WARNING,
+				"IMAGE: get_reserved_mem: %s[%d] has no vaddr",
+				name, instance);
 		return 0;
 	}
 
-	pr_debug("IMAGE: hservice_get_reserved_mem: %s(0x%016lx) address %p",
-			name, range->physaddr, addr);
+	pr_debug(
+		"IMAGE: hservice_get_reserved_mem: %s[%d](0x%016lx) address %p",
+			name, range->instance, range->physaddr,
+			range->buf);
 
-	return (uint64_t)addr + offset;
+	return (uint64_t)range->buf;
 }
 
 void hservice_nanosleep(uint64_t i_seconds, uint64_t i_nano_seconds)
@@ -672,7 +694,7 @@ static int map_hbrt_physmem(struct opal_prd_ctx *ctx, const char *name)
 	struct prd_range *range;
 	void *buf;
 
-	range = find_range(name);
+	range = find_range(name, 0);
 	if (!range) {
 		pr_log(LOG_ERR, "IMAGE: can't find code region %s", name);
 		return -1;
@@ -707,7 +729,12 @@ static void dump_hbrt_map(struct opal_prd_ctx *ctx)
 		return;
 	}
 
-	ftruncate(fd, 0);
+	rc = ftruncate(fd, 0);
+	if (rc < 0) {
+		pr_log(LOG_NOTICE, "IMAGE: couldn't truncate image %s for writing",
+				dump_name);
+		return;
+	}
 	rc = write(fd, ctx->code_addr, ctx->code_size);
 	close(fd);
 
@@ -765,15 +792,32 @@ static int open_and_read(const char *path, void **bufp, int *lenp)
 static int prd_init_one_range(struct opal_prd_ctx *ctx, const char *path,
 		struct dirent *dirent)
 {
-	char *label_path, *reg_path;
+	char *label_path, *reg_path, *instance_path;
 	struct prd_range *range;
 	int label_len, len, rc;
-	char *label;
 	__be64 *reg;
+	char *label;
 	void *buf;
 
-	asprintf(&label_path, "%s/%s/ibm,prd-label", path, dirent->d_name);
-	asprintf(&reg_path, "%s/%s/reg", path, dirent->d_name);
+	rc = asprintf(&label_path, "%s/%s/ibm,prd-label", path, dirent->d_name);
+	if (rc < 0) {
+		pr_log(LOG_ERR, "FW: error creating 'ibm,prd-label' path "
+				"node: %m");
+		return -1;
+	}
+	rc = asprintf(&instance_path, "%s/%s/ibm,prd-instance",
+			path, dirent->d_name);
+	if (rc < 0) {
+		pr_log(LOG_ERR, "FW: error creating 'ibm,prd-instance' path "
+				"node: %m");
+		return -1;
+	}
+	rc = asprintf(&reg_path, "%s/%s/reg", path, dirent->d_name);
+	if (rc < 0) {
+		pr_log(LOG_ERR, "FW: error creating 'reg' path "
+				" node: %m");
+		return -1;
+	}
 
 	reg = NULL;
 	label = NULL;
@@ -808,24 +852,115 @@ static int prd_init_one_range(struct opal_prd_ctx *ctx, const char *path,
 	range->name = strndup(label, label_len);
 	range->physaddr = be64toh(reg[0]);
 	range->size = be64toh(reg[1]);
+	range->buf = NULL;
+	range->multiple = false;
+	range->instance = 0;
+
+	/* optional instance */
+	rc = open_and_read(instance_path, &buf, &len);
+	if (!rc && len == sizeof(uint32_t)) {
+		range->multiple = true;
+		range->instance = be32toh(*(uint32_t *)buf);
+		ctx->fw_range_instances = true;
+	}
 	rc = 0;
 
 out_free:
 	free(reg);
 	free(label);
+	free(instance_path);
 	free(reg_path);
 	free(label_path);
 	return rc;
 }
 
+static int compare_ranges(const void *ap, const void *bp)
+{
+	const struct prd_range *a = ap, *b = bp;
+	int rc;
+
+	rc = strcmp(a->name, b->name);
+	if (rc)
+		return rc;
+
+	if (a->physaddr < b->physaddr)
+		return -1;
+	else if (a->physaddr > b->physaddr)
+		return 1;
+
+	return 0;
+}
+
+static void assign_range_instances(struct opal_prd_ctx *ctx)
+{
+	int i;
+
+	if (!ctx->n_ranges)
+		return;
+
+	ctx->ranges[0].multiple = false;
+	ctx->ranges[0].instance = 0;
+
+	for (i = 1; i < ctx->n_ranges; i++) {
+		struct prd_range *cur, *prev;
+
+		cur = &ctx->ranges[i];
+		prev = &ctx->ranges[i-1];
+
+		if (!strcmp(cur->name, prev->name)) {
+			prev->multiple = true;
+			cur->multiple = true;
+			cur->instance = prev->instance + 1;
+		} else {
+			cur->multiple = false;
+			cur->instance = 0;
+		}
+	}
+}
+
+static void print_ranges(struct opal_prd_ctx *ctx)
+{
+	int i;
+
+	if (ctx->n_ranges == 0)
+		pr_log(LOG_INFO, "FW: No PRD ranges");
+
+	pr_log(LOG_DEBUG, "FW: %d PRD ranges, instances assigned by %s",
+			ctx->n_ranges,
+			ctx->fw_range_instances ? "firmware" : "userspace");
+
+	for (i = 0; i < ctx->n_ranges; i++) {
+		struct prd_range *range = &ctx->ranges[i];
+		char instance_str[20];
+
+		if (range->multiple)
+			snprintf(instance_str, sizeof(instance_str),
+					" [%d]", range->instance);
+		else
+			instance_str[0] = '\0';
+
+		pr_log(LOG_DEBUG, "FW:  %016lx-%016lx %s%s", range->physaddr,
+				range->physaddr + range->size - 1,
+				range->name,
+				instance_str);
+	}
+}
+
 static int prd_init_ranges(struct opal_prd_ctx *ctx)
 {
 	struct dirent *dirent;
 	char *path;
 	DIR *dir;
-	int rc = -1;
+	int rc;
 
-	asprintf(&path, "%s/reserved-memory", devicetree_base);
+	rc = asprintf(&path, "%s/reserved-memory", devicetree_base);
+	if (rc < 0) {
+		pr_log(LOG_ERR, "FW: error creating 'reserved-memory' path "
+				"node: %m");
+		return -1;
+	}
+
+	rc = -1;
 
 	dir = opendir(path);
 	if (!dir) {
@@ -843,6 +978,15 @@ static int prd_init_ranges(struct opal_prd_ctx *ctx)
 	}
 
 	rc = 0;
+	/* sort ranges and assign instance numbers for duplicates (if the
+	 * firmware doesn't number instances for us) */
+	qsort(ctx->ranges, ctx->n_ranges, sizeof(struct prd_range),
+			compare_ranges);
+
+	if (!ctx->fw_range_instances)
+		assign_range_instances(ctx);
+
+	print_ranges(ctx);
 
 out_free:
 	free(path);
@@ -895,7 +1039,7 @@ static int handle_msg_attn(struct opal_prd_ctx *ctx, struct opal_prd_msg *msg)
 
 	rc = call_handle_attns(proc, ipoll_status, ipoll_mask);
 	if (rc) {
-		pr_log(LOG_ERR, "HBRT: enable_attns(%lx,%lx,%lx) failed, rc %d",
+		pr_log(LOG_ERR, "HBRT: handle_attns(%lx,%lx,%lx) failed, rc %d",
 				proc, ipoll_status, ipoll_mask, rc);
 		return -1;
 	}
@@ -1336,11 +1480,18 @@ static void usage(const char *progname)
 "\t--stdio            log to stdio, instead of syslog\n");
 }
 
+static void print_version(void)
+{
+	extern const char version[];
+	printf("opal-prd %s\n", version);
+}
+
 static struct option opal_diag_options[] = {
 	{"file", required_argument, NULL, 'f'},
 	{"pnor", required_argument, NULL, 'p'},
 	{"debug", no_argument, NULL, 'd'},
 	{"help", no_argument, NULL, 'h'},
+	{"version", no_argument, NULL, 'v'},
 	{"stdio", no_argument, NULL, 's'},
 	{ 0 },
 };
@@ -1401,6 +1552,9 @@ int main(int argc, char *argv[])
 		case 'h':
 			usage(argv[0]);
 			return EXIT_SUCCESS;
+		case 'v':
+			print_version();
+			return EXIT_SUCCESS;
 		case '?':
 		default:
 			usage(argv[0]);
