@@ -16,6 +16,7 @@
 
 #include <libflash/libflash.h>
 #include <libflash/libffs.h>
+#include <libflash/blocklevel.h>
 #include "progress.h"
 #include "io.h"
 #include "ast.h"
@@ -23,12 +24,15 @@
 
 #define __aligned(x)			__attribute__((aligned(x)))
 
-#define PFLASH_VERSION	"0.8.6"
+/* Full pflash version number (possibly includes gitid). */
+extern const char version[];
 
 static bool must_confirm = true;
 static bool dummy_run;
 static bool need_relock;
 static bool bmc_flash;
+static uint32_t ffs_toc = 0;
+static int flash_side = 0;
 #ifdef __powerpc__
 static bool using_sfc;
 #endif
@@ -36,8 +40,8 @@ static bool using_sfc;
 #define FILE_BUF_SIZE	0x10000
 static uint8_t file_buf[FILE_BUF_SIZE] __aligned(0x1000);
 
+static struct blocklevel_device *bl;
 static struct spi_flash_ctrl	*fl_ctrl;
-static struct flash_chip	*fl_chip;
 static struct ffs_handle	*ffsh;
 static uint32_t			fl_total_size, fl_erase_granule;
 static const char		*fl_name;
@@ -69,11 +73,54 @@ static void check_confirm(void)
 	must_confirm = false;
 }
 
-static void print_flash_info(void)
+static void print_ffs_info(uint32_t toc_offset)
 {
-	uint32_t i;
+	struct ffs_handle *ffs_handle;
+	uint32_t other_side_offset = 0;
 	int rc;
+	uint32_t i;
 
+	rc = ffs_init(toc_offset, fl_total_size, bl, &ffs_handle, 0);
+	if (rc) {
+		fprintf(stderr, "Error %d opening ffs !\n", rc);
+		return;
+	}
+
+	printf("\n");
+	printf("TOC@0x%08x Partitions:\n", toc_offset);
+	printf("-----------\n");
+
+	for (i = 0;; i++) {
+		uint32_t start, size, act, end;
+		bool ecc;
+		char *name;
+
+		rc = ffs_part_info(ffs_handle, i, &name, &start, &size, &act, &ecc);
+		if (rc == FFS_ERR_PART_NOT_FOUND)
+			break;
+		if (rc) {
+			fprintf(stderr, "Error %d scanning partitions\n", rc);
+			break;
+		}
+		end = start + size;
+		printf("ID=%02d %15s %08x..%08x (actual=%08x) %s\n",
+		       i, name, start, end, act, ecc ? "[ECC]" : "");
+
+		if (strcmp(name, "OTHER_SIDE") == 0)
+			other_side_offset = start;
+
+		free(name);
+	}
+
+	ffs_close(ffs_handle);
+
+	if (other_side_offset)
+		print_ffs_info(other_side_offset);
+}
+
+
+static void print_flash_info(uint32_t toc)
+{
 	printf("Flash info:\n");
 	printf("-----------\n");
 	printf("Name          = %s\n", fl_name);
@@ -83,49 +130,23 @@ static void print_flash_info(void)
 	if (bmc_flash)
 		return;
 
-	if (!ffsh) {
-		rc = ffs_open_flash(fl_chip, 0, 0, &ffsh);
-		if (rc) {
-			fprintf(stderr, "Error %d opening ffs !\n", rc);
-			ffsh = NULL;
-		}
-	}
-	if (!ffsh)
-		return;
-
-	printf("\n");
-	printf("Partitions:\n");
-	printf("-----------\n");
-
-	for (i = 0;; i++) {
-		uint32_t start, size, act, end;
-		char *name;
-
-		rc = ffs_part_info(ffsh, i, &name, &start, &size, &act, NULL);
-		if (rc == FFS_ERR_PART_NOT_FOUND)
-			break;
-		if (rc) {
-			fprintf(stderr, "Error %d scanning partitions\n", rc);
-			break;
-		}
-		end = start + size;
-		printf("ID=%02d %15s %08x..%08x (actual=%08x)\n",
-		       i, name, start, end, act);
-		free(name);
-	}
+	print_ffs_info(toc);
 }
 
-static void lookup_partition(const char *name)
+static int open_partition(const char *name)
 {
 	uint32_t index;
 	int rc;
 
 	/* Open libffs if needed */
 	if (!ffsh) {
-		rc = ffs_open_flash(fl_chip, 0, 0, &ffsh);
+		rc = ffs_init(ffs_toc, fl_total_size, bl, &ffsh, 0);
 		if (rc) {
 			fprintf(stderr, "Error %d opening ffs !\n", rc);
-			exit(1);
+			if (ffs_toc)
+				fprintf(stderr, "You specified 0x%08x as the libffs TOC"
+						" looks like it doesn't exist\n", ffs_toc);
+			return rc;
 		}
 	}
 
@@ -133,14 +154,46 @@ static void lookup_partition(const char *name)
 	rc = ffs_lookup_part(ffsh, name, &index);
 	if (rc == FFS_ERR_PART_NOT_FOUND) {
 		fprintf(stderr, "Partition '%s' not found !\n", name);
-		exit(1);
+		return rc;
 	}
 	if (rc) {
 		fprintf(stderr, "Error %d looking for partition '%s' !\n",
 			rc, name);
-		exit(1);
+		return rc;
 	}
+
 	ffs_index = index;
+	return 0;
+}
+
+static void lookup_partition(const char *name)
+{
+	int rc;
+
+	if (flash_side == 1) {
+		uint32_t other_side_offset;
+
+		rc = open_partition("OTHER_SIDE");
+		if (rc == FFS_ERR_PART_NOT_FOUND)
+			fprintf(stderr, "side 1 was specified but there doesn't appear"
+					" to be a second side to this flash\n");
+		if (rc)
+			exit(1);
+
+		/* Just need to know where it starts */
+		rc = ffs_part_info(ffsh, ffs_index, NULL, &other_side_offset, NULL, NULL, NULL);
+		if (rc)
+			exit(1);
+
+		ffs_close(ffsh);
+		ffsh = NULL;
+
+		ffs_toc = other_side_offset;
+	}
+
+	rc = open_partition(name);
+	if (rc)
+		exit(1);
 }
 
 static void erase_chip(void)
@@ -158,7 +211,7 @@ static void erase_chip(void)
 		return;
 	}
 
-	rc = flash_erase_chip(fl_chip);
+	rc = flash_erase_chip(bl);
 	if (rc) {
 		fprintf(stderr, "Error %d erasing chip\n", rc);
 		exit(1);
@@ -185,7 +238,7 @@ static void erase_range(uint32_t start, uint32_t size, bool will_program)
 	while(size) {
 		/* If aligned to 64k and at least 64k, use 64k erase */
 		if ((start & 0xffff) == 0 && size >= 0x10000) {
-			rc = flash_erase(fl_chip, start, 0x10000);
+			rc = blocklevel_erase(bl, start, 0x10000);
 			if (rc) {
 				fprintf(stderr, "Error %d erasing 0x%08x\n",
 					rc, start);
@@ -195,7 +248,7 @@ static void erase_range(uint32_t start, uint32_t size, bool will_program)
 			size -= 0x10000;
 			done += 0x10000;
 		} else {
-			rc = flash_erase(fl_chip, start, 0x1000);
+			rc = blocklevel_erase(bl, start, 0x1000);
 			if (rc) {
 				fprintf(stderr, "Error %d erasing 0x%08x\n",
 					rc, start);
@@ -218,6 +271,25 @@ static void erase_range(uint32_t start, uint32_t size, bool will_program)
 	}
 }
 
+static void set_ecc(uint32_t start, uint32_t size)
+{
+	uint32_t i = start + 8;
+	uint8_t ecc = 0;
+
+	printf("About to erase and set ECC bits in region 0x%08x to 0x%08x\n", start, start + size);
+	check_confirm();
+	erase_range(start, size, true);
+
+	printf("Programming ECC bits...\n");
+	progress_init(size);
+	while (i < start + size) {
+		blocklevel_write(bl, i, &ecc, sizeof(ecc));
+		i += 9;
+		progress_tick(i - start);
+	}
+	progress_end();
+}
+
 static void program_file(const char *file, uint32_t start, uint32_t size)
 {
 	int fd, rc;
@@ -230,7 +302,7 @@ static void program_file(const char *file, uint32_t start, uint32_t size)
 		exit(1);
 	}
 	printf("About to program \"%s\" at 0x%08x..0x%08x !\n",
-	       file, start, size);
+	       file, start, start + size);
 	check_confirm();
 
 	if (dummy_run) {
@@ -252,7 +324,7 @@ static void program_file(const char *file, uint32_t start, uint32_t size)
 			len = size;
 		size -= len;
 		actual_size += len;
-		rc = flash_write(fl_chip, start, file_buf, len, true);
+		rc = blocklevel_write(bl, start, file_buf, len);
 		if (rc) {
 			if (rc == FLASH_ERR_VERIFY_FAILURE)
 				fprintf(stderr, "Verification failed for"
@@ -287,12 +359,12 @@ static void do_read_file(const char *file, uint32_t start, uint32_t size)
 		exit(1);
 	}
 	printf("Reading to \"%s\" from 0x%08x..0x%08x !\n",
-	       file, start, size);
+	       file, start, start + size);
 
 	progress_init(size >> 8);
 	while(size) {
 		len = size > FILE_BUF_SIZE ? FILE_BUF_SIZE : size;
-		rc = flash_read(fl_chip, start, file_buf, len);
+		rc = blocklevel_read(bl, start, file_buf, len);
 		if (rc) {
 			fprintf(stderr, "Flash read error %d for"
 				" chunk at 0x%08x\n", rc, start);
@@ -318,7 +390,7 @@ static void enable_4B_addresses(void)
 
 	printf("Switching to 4-bytes address mode\n");
 
-	rc = flash_force_4b_mode(fl_chip, true);
+	rc = flash_force_4b_mode(bl, true);
 	if (rc) {
 		fprintf(stderr, "Error %d enabling 4b mode\n", rc);
 		exit(1);
@@ -331,7 +403,7 @@ static void disable_4B_addresses(void)
 
 	printf("Switching to 3-bytes address mode\n");
 
-	rc = flash_force_4b_mode(fl_chip, false);
+	rc = flash_force_4b_mode(bl, false);
 	if (rc) {
 		fprintf(stderr, "Error %d disabling 4b mode\n", rc);
 		exit(1);
@@ -342,7 +414,7 @@ static void flash_access_cleanup_bmc(void)
 {
 	if (ffsh)
 		ffs_close(ffsh);
-	flash_exit(fl_chip);
+	flash_exit(bl);
 	ast_sf_close(fl_ctrl);
 	close_devs();
 }
@@ -362,7 +434,7 @@ static void flash_access_setup_bmc(bool use_lpc, bool need_write)
 	}
 
 	/* Open flash chip */
-	rc = flash_init(fl_ctrl, &fl_chip);
+	rc = flash_init(fl_ctrl, &bl);
 	if (rc) {
 		fprintf(stderr, "Failed to open flash chip\n");
 		exit(1);
@@ -380,7 +452,7 @@ static void flash_access_cleanup_pnor(void)
 
 	if (ffsh)
 		ffs_close(ffsh);
-	flash_exit(fl_chip);
+	flash_exit(bl);
 #ifdef __powerpc__
 	if (using_sfc)
 		sfc_close(fl_ctrl);
@@ -421,7 +493,7 @@ static void flash_access_setup_pnor(bool use_lpc, bool use_sfc, bool need_write)
 #endif
 
 	/* Open flash chip */
-	rc = flash_init(fl_ctrl, &fl_chip);
+	rc = flash_init(fl_ctrl, &bl);
 	if (rc) {
 		fprintf(stderr, "Failed to open flash chip\n");
 		exit(1);
@@ -437,7 +509,7 @@ static void flash_access_setup_pnor(bool use_lpc, bool use_sfc, bool need_write)
 
 static void print_version(void)
 {
-	printf("Palmetto Flash tool " PFLASH_VERSION "\n");
+	printf("Open-Power Flash tool %s\n", version);
 }
 
 static void print_help(const char *pname)
@@ -465,6 +537,11 @@ static void print_help(const char *pname)
 #endif
 	printf("\t-b, --bmc\n");
 	printf("\t\tTarget BMC flash instead of host flash\n\n");
+	printf("\t-S, --side\n");
+	printf("\t\tSide of the flash on which to operate, 0 (default) or 1\n\n");
+	printf("\t-T, --toc\n");
+	printf("\t\tlibffs TOC on which to operate, defaults to 0.\n");
+	printf("\t\tleading 0x is required for interpretation of a hex value\n\n");
 	printf(" Commands:\n");
 	printf("\t-4, --enable-4B\n");
 	printf("\t\tSwitch the flash and controller to 4-bytes address\n");
@@ -496,6 +573,10 @@ static void print_help(const char *pname)
 	printf("\t-t, --tune\n");
 	printf("\t\tJust tune the flash controller & access size\n");
 	printf("\t\t(Implicit for all other operations)\n\n");
+	printf("\t-c --clear\n");
+	printf("\t\tUsed to ECC clear a partition of the flash\n");
+	printf("\t\tMust be used in conjunction with -P. Will erase the\n");
+	printf("\t\tpartition and then set all the ECC bits as they should be\n\n");
 	printf("\t-i, --info\n");
 	printf("\t\tDisplay some information about the flash.\n\n");
 	printf("\t-h, --help\n");
@@ -507,13 +588,14 @@ int main(int argc, char *argv[])
 	const char *pname = argv[0];
 	uint32_t address = 0, read_size = 0, write_size = 0;
 	uint32_t erase_start = 0, erase_size = 0;
-	bool erase = false;
+	bool erase = false, do_clear = false;
 	bool program = false, erase_all = false, info = false, do_read = false;
 	bool enable_4B = false, disable_4B = false, use_lpc = true;
 	bool show_help = false, show_version = false;
 	bool has_sfc = false, has_ast = false;
 	bool no_action = false, tune = false;
 	char *write_file = NULL, *read_file = NULL, *part_name = NULL;
+	bool ffs_toc_seen = false;
 	int rc;
 
 	while(1) {
@@ -536,10 +618,13 @@ int main(int argc, char *argv[])
 			{"help",	no_argument,		NULL,	'h'},
 			{"version",	no_argument,		NULL,	'v'},
 			{"debug",	no_argument,		NULL,	'g'},
+			{"side",	required_argument,	NULL,	'S'},
+			{"toc",		required_argument,	NULL,	'T'},
+			{"clear",   no_argument,        NULL,   'c'}
 		};
 		int c, oidx = 0;
 
-		c = getopt_long(argc, argv, "a:s:P:r:43Eep:fdihlvbtg",
+		c = getopt_long(argc, argv, "a:s:P:r:43Eep:fdihlvbtgS:T:c",
 				long_opts, &oidx);
 		if (c == EOF)
 			break;
@@ -601,6 +686,16 @@ int main(int argc, char *argv[])
 		case 'g':
 			libflash_debug = true;
 			break;
+		case 'S':
+			flash_side = atoi(optarg);
+			break;
+		case 'T':
+			ffs_toc_seen = true;
+			ffs_toc = strtoul(optarg, NULL, 0);
+			break;
+		case 'c':
+			do_clear = true;
+			break;
 		default:
 			exit(1);
 		}
@@ -610,7 +705,7 @@ int main(int argc, char *argv[])
 	 * also tune them as a side effect
 	 */
 	no_action = !erase && !program && !info && !do_read &&
-		!enable_4B && !disable_4B && !tune;
+		!enable_4B && !disable_4B && !tune && !do_clear;
 
 	/* Nothing to do, if we didn't already, print usage */
 	if (no_action && !show_version)
@@ -668,6 +763,22 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	if (do_clear && !part_name) {
+		fprintf(stderr, "--clear only supported on a partition name\n");
+		exit(1);
+	}
+
+	/* Explicitly only support two sides */
+	if (flash_side != 0 && flash_side != 1) {
+		fprintf(stderr, "Unexpected value for --side '%d'\n", flash_side);
+		exit(1);
+	}
+
+	if (ffs_toc_seen && flash_side) {
+		fprintf(stderr, "--toc and --side are exclusive");
+		exit(1);
+	}
+
 	/* If file specified but not size, get size from file
 	 */
 	if (write_file && !write_size) {
@@ -698,7 +809,7 @@ int main(int argc, char *argv[])
 		flash_access_setup_pnor(use_lpc, has_sfc, erase || program);
 	}
 
-	rc = flash_get_info(fl_chip, &fl_name,
+	rc = blocklevel_get_info(bl, &fl_name,
 			    &fl_total_size, &fl_erase_granule);
 	if (rc) {
 		fprintf(stderr, "Error %d getting flash info\n", rc);
@@ -720,15 +831,24 @@ int main(int argc, char *argv[])
 	/* We have a partition, adjust read/write size if needed */
 	if (ffsh && ffs_index >= 0) {
 		uint32_t pstart, pmaxsz, pactsize;
+		bool ecc;
 		int rc;
 
 		rc = ffs_part_info(ffsh, ffs_index, NULL,
-				   &pstart, &pmaxsz, &pactsize, NULL);
+				   &pstart, &pmaxsz, &pactsize, &ecc);
 		if (rc) {
 			fprintf(stderr,"Failed to get partition info\n");
 			exit(1);
 		}
 
+		if (!ecc && do_clear) {
+			fprintf(stderr, "The partition on which to do --clear "
+					"does not have ECC, are you sure?\n");
+			check_confirm();
+			/* Still confirm later on */
+			must_confirm = true;
+		}
+
 		/* Read size is obtained from partition "actual" size */
 		if (!read_size)
 			read_size = pactsize;
@@ -778,8 +898,14 @@ int main(int argc, char *argv[])
 		enable_4B_addresses();
 	if (disable_4B)
 		disable_4B_addresses();
-	if (info)
-		print_flash_info();
+	if (info ) {
+		/*
+		 * Don't pass through modfied TOC value if the modification was done
+		 * because of --size, but still respect if it came from --toc (we
+		 * assume the user knows what they're doing in that case)
+		 */
+		print_flash_info(flash_side ? 0 : ffs_toc);
+	}
 	if (do_read)
 		do_read_file(read_file, address, read_size);
 	if (erase_all)
@@ -788,6 +914,7 @@ int main(int argc, char *argv[])
 		erase_range(erase_start, erase_size, program);
 	if (program)
 		program_file(write_file, address, write_size);
-
+	if (do_clear)
+		set_ecc(address, write_size);
 	return 0;
 }
