@@ -116,16 +116,16 @@ static int64_t fsp_dump_read(void);
 DEFINE_LOG_ENTRY(OPAL_RC_DUMP_INIT, OPAL_PLATFORM_ERR_EVT, OPAL_DUMP,
 		 OPAL_PLATFORM_FIRMWARE,
 		 OPAL_PREDICTIVE_ERR_FAULT_RECTIFY_REBOOT,
-		 OPAL_NA, NULL);
+		 OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_DUMP_LIST, OPAL_PLATFORM_ERR_EVT, OPAL_DUMP,
 		 OPAL_PLATFORM_FIRMWARE,
 		 OPAL_INFO,
-		 OPAL_NA, NULL);
+		 OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_DUMP_ACK, OPAL_PLATFORM_ERR_EVT, OPAL_DUMP,
 		 OPAL_PLATFORM_FIRMWARE, OPAL_INFO,
-		 OPAL_NA, NULL);
+		 OPAL_NA);
 
 /*
  * Helper functions
