@@ -18,6 +18,8 @@
 /*
  * LED location code and indicator handling
  */
+
+#define pr_fmt(fmt) "FSPLED: " fmt
 #include <skiboot.h>
 #include <fsp.h>
 #include <device.h>
@@ -26,11 +28,8 @@
 #include <errorlog.h>
 #include <opal.h>
 #include <opal-msg.h>
-
-#include "fsp-leds.h"
-
-/* LED prefix */
-#define PREFIX		"FSPLED: "
+#include <fsp-leds.h>
+#include <fsp-sysparam.h>
 
 #define buf_write(p, type, val)  do { *(type *)(p) = val;\
 					p += sizeof(type); } while(0)
@@ -71,6 +70,7 @@ static struct list_head  spcn_cmdq;	/* SPCN command queue */
 /* LED lock */
 static struct lock led_lock = LOCK_UNLOCKED;
 static struct lock spcn_cmd_lock = LOCK_UNLOCKED;
+static struct lock sai_lock = LOCK_UNLOCKED;
 
 static bool spcn_cmd_complete = true;	/* SPCN command complete */
 
@@ -78,6 +78,13 @@ static bool spcn_cmd_complete = true;	/* SPCN command complete */
 static u32 last_spcn_cmd;
 static int replay = 0;
 
+/*
+ * FSP controls System Attention Indicator. But it expects hypervisor
+ * keep track of the status and serve get LED state request (both from
+ * Linux and FSP itself)!
+ */
+static struct sai_data sai_data;
+
 /* Forward declaration */
 static void fsp_read_leds_data_complete(struct fsp_msg *msg);
 static int process_led_state_change(void);
@@ -85,21 +92,21 @@ static int process_led_state_change(void);
 
 DEFINE_LOG_ENTRY(OPAL_RC_LED_SPCN, OPAL_PLATFORM_ERR_EVT, OPAL_LED,
 		OPAL_PLATFORM_FIRMWARE, OPAL_PREDICTIVE_ERR_GENERAL,
-		OPAL_NA, NULL);
+		OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_LED_BUFF, OPAL_PLATFORM_ERR_EVT, OPAL_LED,
 		OPAL_PLATFORM_FIRMWARE, OPAL_PREDICTIVE_ERR_GENERAL,
-		OPAL_NA, NULL);
+		OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_LED_LC, OPAL_PLATFORM_ERR_EVT, OPAL_LED,
-		OPAL_PLATFORM_FIRMWARE, OPAL_INFO, OPAL_NA, NULL);
+		OPAL_PLATFORM_FIRMWARE, OPAL_INFO, OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_LED_STATE, OPAL_PLATFORM_ERR_EVT, OPAL_LED,
 		OPAL_PLATFORM_FIRMWARE, OPAL_PREDICTIVE_ERR_GENERAL,
-		OPAL_NA, NULL);
+		OPAL_NA);
 
 DEFINE_LOG_ENTRY(OPAL_RC_LED_SUPPORT, OPAL_PLATFORM_ERR_EVT, OPAL_LED,
-		OPAL_PLATFORM_FIRMWARE, OPAL_INFO, OPAL_NA, NULL);
+		OPAL_PLATFORM_FIRMWARE, OPAL_INFO, OPAL_NA);
 
 
 /* Find descendent LED record with CEC location code in CEC list */
@@ -195,6 +202,152 @@ static inline void opal_led_update_complete(u64 async_token, u64 result)
 	opal_queue_msg(OPAL_MSG_ASYNC_COMP, NULL, NULL, async_token, result);
 }
 
+static inline bool is_sai_loc_code(char *loc_code)
+{
+	if (!strcmp(sai_data.loc_code, loc_code))
+		return true;
+
+	return false;
+}
+
+/* Set/Reset System attention indicator */
+static void fsp_set_sai_complete(struct fsp_msg *msg)
+{
+	int ret = OPAL_SUCCESS;
+	int rc = msg->resp->word1 & 0xff00;
+	struct led_set_cmd *spcn_cmd = (struct led_set_cmd *)msg->user_data;
+
+	if (rc) {
+		prlog(PR_ERR, "Update SAI cmd failed [rc=%d].\n", rc);
+		ret = OPAL_INTERNAL_ERROR;
+
+		/* Roll back */
+		lock(&sai_lock);
+		sai_data.state = spcn_cmd->ckpt_status;
+		unlock(&sai_lock);
+	}
+
+	if (spcn_cmd->cmd_src == SPCN_SRC_OPAL)
+		opal_led_update_complete(spcn_cmd->async_token, ret);
+
+	/* free msg and spcn command */
+	free(spcn_cmd);
+	fsp_freemsg(msg);
+
+	/* Process pending LED update request */
+	process_led_state_change();
+}
+
+static int fsp_set_sai(struct led_set_cmd *spcn_cmd)
+{
+	int rc = -ENOMEM;
+	uint32_t cmd = FSP_CMD_SA_INDICATOR;
+	struct fsp_msg *msg;
+
+	/*
+	 * FSP does not allow hypervisor to set real SAI, but we can
+	 * reset real SAI. Also in our case only host can control
+	 * LEDs, not guests. Hence we will set platform virtual SAI
+	 * and reset real SAI.
+	 */
+	if (spcn_cmd->state == LED_STATE_ON)
+		cmd |= FSP_LED_SET_PLAT_SAI;
+	else
+		cmd |= FSP_LED_RESET_REAL_SAI;
+
+	prlog(PR_TRACE, "Update SAI Indicator [cur : 0x%x, new : 0x%x].\n",
+	      sai_data.state, spcn_cmd->state);
+
+	msg = fsp_mkmsg(cmd, 0);
+	if (!msg) {
+		prlog(PR_ERR, "%s: Memory allocation failed.\n", __func__);
+		goto sai_fail;
+	}
+
+	spcn_cmd->ckpt_status = sai_data.state;
+	msg->user_data = spcn_cmd;
+	rc = fsp_queue_msg(msg, fsp_set_sai_complete);
+	if (rc) {
+		fsp_freemsg(msg);
+		prlog(PR_ERR, "%s: Failed to queue the message\n", __func__);
+		goto sai_fail;
+	}
+
+	lock(&sai_lock);
+	sai_data.state = spcn_cmd->state;
+	unlock(&sai_lock);
+
+	return OPAL_SUCCESS;
+
+sai_fail:
+	if (spcn_cmd->cmd_src == SPCN_SRC_OPAL)
+		opal_led_update_complete(spcn_cmd->async_token,
+					 OPAL_INTERNAL_ERROR);
+
+	return OPAL_INTERNAL_ERROR;
+}
+
+static void fsp_get_sai_complete(struct fsp_msg *msg)
+{
+	int rc = msg->resp->word1 & 0xff00;
+
+	if (rc) {
+		prlog(PR_ERR, "Read real SAI cmd failed [rc = 0x%x].\n", rc);
+	} else { /* Update SAI state */
+		lock(&sai_lock);
+		sai_data.state = msg->resp->data.words[0] & 0xff;
+		unlock(&sai_lock);
+
+		prlog(PR_TRACE, "SAI initial state = 0x%x\n", sai_data.state);
+	}
+
+	fsp_freemsg(msg);
+}
+
+/* Read initial SAI state. */
+static void fsp_get_sai(void)
+{
+	int rc;
+	uint32_t cmd = FSP_CMD_SA_INDICATOR | FSP_LED_READ_REAL_SAI;
+	struct fsp_msg *msg;
+
+	msg = fsp_mkmsg(cmd, 0);
+	if (!msg) {
+		prlog(PR_ERR, "%s: Memory allocation failed.\n", __func__);
+		return;
+	}
+	rc = fsp_queue_msg(msg, fsp_get_sai_complete);
+	if (rc) {
+		fsp_freemsg(msg);
+		prlog(PR_ERR, "%s: Failed to queue the message\n", __func__);
+	}
+}
+
+static bool sai_update_notification(struct fsp_msg *msg)
+{
+	uint32_t *state = &msg->data.words[2];
+	uint32_t param_id = msg->data.words[0];
+	int len = msg->data.words[1] & 0xffff;
+
+	if (param_id != SYS_PARAM_REAL_SAI && param_id != SYS_PARAM_PLAT_SAI)
+		return false;
+
+	if ( len != 4)
+		return false;
+
+	if (*state != LED_STATE_ON && *state != LED_STATE_OFF)
+		return false;
+
+	/* Update SAI state */
+	lock(&sai_lock);
+	sai_data.state = *state;
+	unlock(&sai_lock);
+
+	prlog(PR_TRACE, "SAI updated. New SAI state = 0x%x\n", *state);
+	return true;
+}
+
+
 /*
  * Update both the local LED lists to reflect upon led state changes
  * occured with the recent SPCN command. Subsequent LED requests will
@@ -209,7 +362,7 @@ static void update_led_list(char *loc_code, u32 led_state, u32 excl_bit)
 	encl_cec_led = fsp_find_encl_cec_led(loc_code);
 	if (!encl_cec_led) {
 		log_simple_error(&e_info(OPAL_RC_LED_LC),
-			PREFIX "Could not find enclosure LED in CEC LC=%s\n",
+			"Could not find enclosure LED in CEC LC=%s\n",
 			loc_code);
 		return;
 	}
@@ -221,7 +374,7 @@ static void update_led_list(char *loc_code, u32 led_state, u32 excl_bit)
 	} else {	/* Descendant LED in CEC list */
 		led = fsp_find_cec_led(loc_code);
 		if (!led) {
-			log_simple_error(&e_info(OPAL_RC_LED_LC), PREFIX
+			log_simple_error(&e_info(OPAL_RC_LED_LC),
 					 "Could not find descendent LED in \
 					 CEC LC=%s\n", loc_code);
 			return;
@@ -233,7 +386,7 @@ static void update_led_list(char *loc_code, u32 led_state, u32 excl_bit)
 	encl_led = fsp_find_encl_encl_led(loc_code);
 	if (!encl_led) {
 		log_simple_error(&e_info(OPAL_RC_LED_LC),
-			PREFIX "Could not find enclosure LED in ENCL LC=%s\n",
+			"Could not find enclosure LED in ENCL LC=%s\n",
 			loc_code);
 		return;
 	}
@@ -260,15 +413,14 @@ static int fsp_set_led_response(uint32_t cmd)
 
 	msg = fsp_mkmsg(cmd, 0);
 	if (!msg) {
-		prerror(PREFIX
-			"Failed to allocate FSP_RSP_SET_LED_STATE [cmd=%x])\n",
+		prerror("Failed to allocate FSP_RSP_SET_LED_STATE [cmd=%x])\n",
 			cmd);
 	} else {
 		rc = fsp_queue_msg(msg, fsp_freemsg);
 		if (rc != OPAL_SUCCESS) {
 			fsp_freemsg(msg);
-			prerror(PREFIX "Failed to queue "
-				"FSP_RSP_SET_LED_STATE [cmd=%x]\n", cmd);
+			prerror("Failed to queue FSP_RSP_SET_LED_STATE"
+				" [cmd=%x]\n", cmd);
 		}
 	}
 	return rc;
@@ -291,7 +443,7 @@ static void fsp_spcn_set_led_completion(struct fsp_msg *msg)
 	 */
 	if (status != FSP_STATUS_SUCCESS) {
 		log_simple_error(&e_info(OPAL_RC_LED_SPCN),
-			PREFIX "Last SPCN command failed, status=%02x\n",
+			"Last SPCN command failed, status=%02x\n",
 			status);
 		cmd |= FSP_STATUS_GENERIC_ERROR;
 
@@ -451,7 +603,7 @@ static int fsp_msg_set_led_state(struct led_set_cmd *spcn_cmd)
 update_fail:
 	if (rc) {
 		log_simple_error(&e_info(OPAL_RC_LED_STATE),
-				 PREFIX "Set led state failed at LC=%s\n",
+				 "Set led state failed at LC=%s\n",
 				 spcn_cmd->loc_code);
 
 		if (spcn_cmd->cmd_src == SPCN_SRC_FSP)
@@ -493,7 +645,11 @@ static int process_led_state_change(void)
 	spcn_cmd = list_pop(&spcn_cmdq, struct led_set_cmd, link);
 	unlock(&spcn_cmd_lock);
 
-	rc = fsp_msg_set_led_state(spcn_cmd);
+	if (is_sai_loc_code(spcn_cmd->loc_code))
+		rc = fsp_set_sai(spcn_cmd);
+	else
+		rc = fsp_msg_set_led_state(spcn_cmd);
+
 	if (rc) {
 		free(spcn_cmd);
 		process_led_state_change();
@@ -522,13 +678,12 @@ static int queue_led_state_change(char *loc_code, u8 command,
 	/* New request node */
 	cmd = zalloc(sizeof(struct led_set_cmd));
 	if (!cmd) {
-		prlog(PR_ERR, PREFIX
-		      "SPCN set command node allocation failed\n");
+		prlog(PR_ERR, "SPCN set command node allocation failed\n");
 		return -1;
 	}
 
 	/* Save the request */
-	strncpy(cmd->loc_code, loc_code, strlen(loc_code));
+	strncpy(cmd->loc_code, loc_code, LOC_CODE_SIZE - 1);
 	cmd->command = command;
 	cmd->state = state;
 	cmd->cmd_src = cmd_src;
@@ -691,12 +846,11 @@ static void fsp_ret_loc_code_list(u16 req_type, char *loc_code)
 	msg = fsp_mkmsg(FSP_RSP_GET_LED_LIST, 3, 0,
 			PSI_DMA_LOC_COD_BUF, total_size);
 	if (!msg) {
-		prerror(PREFIX "Failed to allocate FSP_RSP_GET_LED_LIST.\n");
+		prerror("Failed to allocate FSP_RSP_GET_LED_LIST.\n");
 	} else {
 		if (fsp_queue_msg(msg, fsp_freemsg)) {
 			fsp_freemsg(msg);
-			prerror(PREFIX
-				"Failed to queue FSP_RSP_GET_LED_LIST\n");
+			prerror("Failed to queue FSP_RSP_GET_LED_LIST\n");
 		}
 	}
 }
@@ -728,12 +882,12 @@ static void fsp_get_led_list(struct fsp_msg *msg)
 		msg = fsp_mkmsg(FSP_RSP_GET_LED_LIST | FSP_STATUS_INVALID_DATA,
 				0);
 		if (!msg) {
-			prerror(PREFIX "Failed to allocate FSP_RSP_GET_LED_LIST"
+			prerror("Failed to allocate FSP_RSP_GET_LED_LIST"
 				" | FSP_STATUS_INVALID_DATA\n");
 		} else {
 			if (fsp_queue_msg(msg, fsp_freemsg)) {
 				fsp_freemsg(msg);
-				prerror(PREFIX "Failed to queue "
+				prerror("Failed to queue "
 					"FSP_RSP_GET_LED_LIST |"
 					" FSP_STATUS_INVALID_DATA\n");
 			}
@@ -742,7 +896,7 @@ static void fsp_get_led_list(struct fsp_msg *msg)
 	}
 	memcpy(&req, buf, sizeof(req));
 
-	prlog(PR_TRACE, PREFIX "Request for loc code list type 0x%04x LC=%s\n",
+	prlog(PR_TRACE, "Request for loc code list type 0x%04x LC=%s\n",
 	       req.req_type, req.loc_code);
 
 	fsp_ret_loc_code_list(req.req_type, req.loc_code);
@@ -765,18 +919,18 @@ static void fsp_free_led_list_buf(struct fsp_msg *msg)
 	/* Token does not point to outbound buffer */
 	if (tce_token != PSI_DMA_LOC_COD_BUF) {
 		log_simple_error(&e_info(OPAL_RC_LED_BUFF),
-			PREFIX "Invalid tce token from FSP\n");
+			"Invalid tce token from FSP\n");
 		cmd |=  FSP_STATUS_GENERIC_ERROR;
 		resp = fsp_mkmsg(cmd, 0);
 		if (!resp) {
-			prerror(PREFIX "Failed to allocate FSP_RSP_RET_LED_BUFFER"
+			prerror("Failed to allocate FSP_RSP_RET_LED_BUFFER"
 				"| FSP_STATUS_GENERIC_ERROR\n");
 			return;
 		}
 
 		if (fsp_queue_msg(resp, fsp_freemsg)) {
 			fsp_freemsg(resp);
-			prerror(PREFIX "Failed to queue "
+			prerror("Failed to queue "
 				"RET_LED_BUFFER|ERROR\n");
 		}
 		return;
@@ -787,58 +941,61 @@ static void fsp_free_led_list_buf(struct fsp_msg *msg)
 
 	resp = fsp_mkmsg(cmd, 0);
 	if (!resp) {
-		prerror(PREFIX "Failed to allocate FSP_RSP_RET_LED_BUFFER\n");
+		prerror("Failed to allocate FSP_RSP_RET_LED_BUFFER\n");
 		return;
 	}
 	if (fsp_queue_msg(resp, fsp_freemsg)) {
 		fsp_freemsg(resp);
-		prerror(PREFIX "Failed to queue FSP_RSP_RET_LED_BUFFER\n");
+		prerror("Failed to queue FSP_RSP_RET_LED_BUFFER\n");
 	}
 }
 
 static void fsp_ret_led_state(char *loc_code)
 {
-	struct fsp_led_data *led, *next;
+	bool found = false;
 	u8 ind_state = 0;
+	u32 cmd = FSP_RSP_GET_LED_STATE;
+	struct fsp_led_data *led, *next;
 	struct fsp_msg *msg;
 
-	list_for_each_safe(&cec_ledq, led, next, link) {
-		if (strcmp(loc_code, led->loc_code))
-			continue;
+	if (is_sai_loc_code(loc_code)) {
+		if (sai_data.state & OPAL_SLOT_LED_STATE_ON)
+			ind_state = FSP_IND_FAULT_ACTV;
+		found = true;
+	} else {
+		list_for_each_safe(&cec_ledq, led, next, link) {
+			if (strcmp(loc_code, led->loc_code))
+				continue;
 
-		/* Found the location code */
-		if (led->status & SPCN_LED_IDENTIFY_MASK)
-			ind_state |= FSP_IND_IDENTIFY_ACTV;
-		if (led->status & SPCN_LED_FAULT_MASK)
-			ind_state |= FSP_IND_FAULT_ACTV;
-		msg = fsp_mkmsg(FSP_RSP_GET_LED_STATE, 1, ind_state);
-		if (!msg) {
-			prerror(PREFIX
-				"Couldn't alloc FSP_RSP_GET_LED_STATE\n");
-			return;
+			/* Found the location code */
+			if (led->status & SPCN_LED_IDENTIFY_MASK)
+				ind_state |= FSP_IND_IDENTIFY_ACTV;
+			if (led->status & SPCN_LED_FAULT_MASK)
+				ind_state |= FSP_IND_FAULT_ACTV;
+
+			found = true;
+			break;
 		}
-		if (fsp_queue_msg(msg, fsp_freemsg)) {
-			fsp_freemsg(msg);
-			prerror(PREFIX
-				"Couldn't queue FSP_RSP_GET_LED_STATE\n");
-		}
-		return;
 	}
 
 	/* Location code not found */
-	log_simple_error(&e_info(OPAL_RC_LED_LC),
-		PREFIX "Could not find the location code LC=%s\n", loc_code);
+	if (!found) {
+		log_simple_error(&e_info(OPAL_RC_LED_LC),
+				 "Could not find the location code LC=%s\n",
+				 loc_code);
+		cmd |= FSP_STATUS_INVALID_LC;
+		ind_state = 0xff;
+	}
 
-	msg = fsp_mkmsg(FSP_RSP_GET_LED_STATE | FSP_STATUS_INVALID_LC, 1, 0xff);
+	msg = fsp_mkmsg(cmd, 1, ind_state);
 	if (!msg) {
-		prerror(PREFIX "Failed to alloc FSP_RSP_GET_LED_STATE "
-			"| FSP_STATUS_INVALID_LC\n");
+		prerror("Couldn't alloc FSP_RSP_GET_LED_STATE\n");
 		return;
 	}
+
 	if (fsp_queue_msg(msg, fsp_freemsg)) {
 		fsp_freemsg(msg);
-		prerror(PREFIX "Failed to queue FSP_RSP_GET_LED_STATE "
-			"| FSP_STATUS_INVALID_LC\n");
+		prerror("Couldn't queue FSP_RSP_GET_LED_STATE\n");
 	}
 }
 
@@ -860,13 +1017,13 @@ static void fsp_get_led_state(struct fsp_msg *msg)
 		msg = fsp_mkmsg(FSP_RSP_GET_LED_STATE |
 				FSP_STATUS_INVALID_DATA, 0);
 		if (!msg) {
-			prerror(PREFIX "Failed to allocate FSP_RSP_GET_LED_STATE"
+			prerror("Failed to allocate FSP_RSP_GET_LED_STATE"
 				" | FSP_STATUS_INVALID_DATA\n");
 			return;
 		}
 		if (fsp_queue_msg(msg, fsp_freemsg)) {
 			fsp_freemsg(msg);
-			prerror(PREFIX "Failed to queue FSP_RSP_GET_LED_STATE"
+			prerror("Failed to queue FSP_RSP_GET_LED_STATE"
 				" | FSP_STATUS_INVALID_DATA\n");
 		}
 		return;
@@ -882,8 +1039,8 @@ static void fsp_get_led_state(struct fsp_msg *msg)
 	/* Bound check */
 	if (req.lc_len >= LOC_CODE_SIZE) {
 		log_simple_error(&e_info(OPAL_RC_LED_LC),
-			PREFIX "Loc code too large in %s: %d bytes\n",
-			__func__, req.lc_len);
+				 "Loc code too large in %s: %d bytes\n",
+				 __func__, req.lc_len);
 		req.lc_len = LOC_CODE_SIZE - 1;
 	}
 	/* Ensure NULL termination */
@@ -926,8 +1083,8 @@ static void fsp_set_led_state(struct fsp_msg *msg)
 	/* Bound check */
 	if (req.lc_len >= LOC_CODE_SIZE) {
 		log_simple_error(&e_info(OPAL_RC_LED_LC),
-			PREFIX "Loc code too large in %s: %d bytes\n",
-			__func__, req.lc_len);
+				 "Loc code too large in %s: %d bytes\n",
+				 __func__, req.lc_len);
 		req.lc_len = LOC_CODE_SIZE - 1;
 	}
 	/* Ensure NULL termination */
@@ -982,30 +1139,26 @@ static bool fsp_indicator_message(u32 cmd_sub_mod, struct fsp_msg *msg)
 	/* LED support not available yet */
 	if (led_support != LED_STATE_PRESENT) {
 		log_simple_error(&e_info(OPAL_RC_LED_SUPPORT),
-			PREFIX "Indicator message while LED support not"
+			"Indicator message while LED support not"
 			" available yet\n");
 		return false;
 	}
 
 	switch (cmd_sub_mod) {
 	case FSP_CMD_GET_LED_LIST:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_GET_LED_LIST command received\n");
+		prlog(PR_TRACE, "FSP_CMD_GET_LED_LIST command received\n");
 		fsp_get_led_list(msg);
 		return true;
 	case FSP_CMD_RET_LED_BUFFER:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_RET_LED_BUFFER command received\n");
+		prlog(PR_TRACE, "FSP_CMD_RET_LED_BUFFER command received\n");
 		fsp_free_led_list_buf(msg);
 		return true;
 	case FSP_CMD_GET_LED_STATE:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_GET_LED_STATE command received\n");
+		prlog(PR_TRACE, "FSP_CMD_GET_LED_STATE command received\n");
 		fsp_get_led_state(msg);
 		return true;
 	case FSP_CMD_SET_LED_STATE:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_SET_LED_STATE command received\n");
+		prlog(PR_TRACE, "FSP_CMD_SET_LED_STATE command received\n");
 		fsp_set_led_state(msg);
 		return true;
 	/*
@@ -1014,54 +1167,44 @@ static bool fsp_indicator_message(u32 cmd_sub_mod, struct fsp_msg *msg)
 	 * the field service processor with a generic error.
 	 */
 	case FSP_CMD_GET_MTMS_LIST:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_GET_MTMS_LIST command received\n");
+		prlog(PR_TRACE, "FSP_CMD_GET_MTMS_LIST command received\n");
 		cmd = FSP_RSP_GET_MTMS_LIST;
 		break;
 	case FSP_CMD_RET_MTMS_BUFFER:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_RET_MTMS_BUFFER command received\n");
+		prlog(PR_TRACE, "FSP_CMD_RET_MTMS_BUFFER command received\n");
 		cmd = FSP_RSP_RET_MTMS_BUFFER;
 		break;
 	case FSP_CMD_SET_ENCL_MTMS:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_SET_MTMS command received\n");
+		prlog(PR_TRACE, "FSP_CMD_SET_MTMS command received\n");
 		cmd = FSP_RSP_SET_ENCL_MTMS;
 		break;
 	case FSP_CMD_CLR_INCT_ENCL:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_CLR_INCT_ENCL command received\n");
+		prlog(PR_TRACE, "FSP_CMD_CLR_INCT_ENCL command received\n");
 		cmd = FSP_RSP_CLR_INCT_ENCL;
 		break;
 	case FSP_CMD_ENCL_MCODE_INIT:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_ENCL_MCODE_INIT command received\n");
+		prlog(PR_TRACE, "FSP_CMD_ENCL_MCODE_INIT command received\n");
 		cmd = FSP_RSP_ENCL_MCODE_INIT;
 		break;
 	case FSP_CMD_ENCL_MCODE_INTR:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_ENCL_MCODE_INTR command received\n");
+		prlog(PR_TRACE, "FSP_CMD_ENCL_MCODE_INTR command received\n");
 		cmd = FSP_RSP_ENCL_MCODE_INTR;
 		break;
 	case FSP_CMD_ENCL_POWR_TRACE:
-		prlog(PR_TRACE, PREFIX
-		      "FSP_CMD_ENCL_POWR_TRACE command received\n");
+		prlog(PR_TRACE, "FSP_CMD_ENCL_POWR_TRACE command received\n");
 		cmd = FSP_RSP_ENCL_POWR_TRACE;
 		break;
 	case FSP_CMD_RET_ENCL_TRACE_BUFFER:
-		prlog(PR_TRACE, PREFIX "FSP_CMD_RET_ENCL_TRACE_BUFFER \
-		      command received\n");
+		prlog(PR_TRACE, "FSP_CMD_RET_ENCL_TRACE_BUFFER command received\n");
 		cmd = FSP_RSP_RET_ENCL_TRACE_BUFFER;
 		break;
 	case FSP_CMD_GET_SPCN_LOOP_STATUS:
-		prlog(PR_TRACE, PREFIX "FSP_CMD_GET_SPCN_LOOP_STATUS \
-		      command received\n");
+		prlog(PR_TRACE, "FSP_CMD_GET_SPCN_LOOP_STATUS command received\n");
 		cmd = FSP_RSP_GET_SPCN_LOOP_STATUS;
 		break;
 	case FSP_CMD_INITIATE_LAMP_TEST:
 		/* XXX: FSP ACK not required for this sub command */
-		prlog(PR_TRACE, PREFIX "FSP_CMD_INITIATE_LAMP_TEST \
-		      command received\n");
+		prlog(PR_TRACE, "FSP_CMD_INITIATE_LAMP_TEST command received\n");
 		return true;
 	default:
 		return false;
@@ -1069,14 +1212,12 @@ static bool fsp_indicator_message(u32 cmd_sub_mod, struct fsp_msg *msg)
 	cmd |= FSP_STATUS_GENERIC_ERROR;
 	resp = fsp_mkmsg(cmd, 0);
 	if (!resp) {
-		prerror(PREFIX
-			"Failed to allocate FSP_STATUS_GENERIC_ERROR\n");
+		prerror("Failed to allocate FSP_STATUS_GENERIC_ERROR\n");
 		return false;
 	}
 	if (fsp_queue_msg(resp, fsp_freemsg)) {
 		fsp_freemsg(resp);
-		prerror(PREFIX
-			"Failed to queue FSP_STATUS_GENERIC_ERROR\n");
+		prerror("Failed to queue FSP_STATUS_GENERIC_ERROR\n");
 		return false;
 	}
 	return true;
@@ -1088,6 +1229,31 @@ static struct fsp_client fsp_indicator_client = {
 };
 
 
+static int fsp_opal_get_sai(u64 *led_mask, u64 *led_value)
+{
+	*led_mask |= OPAL_SLOT_LED_STATE_ON << OPAL_SLOT_LED_TYPE_ATTN;
+	if (sai_data.state & OPAL_SLOT_LED_STATE_ON)
+		*led_value |=
+			OPAL_SLOT_LED_STATE_ON << OPAL_SLOT_LED_TYPE_ATTN;
+
+	return OPAL_SUCCESS;
+}
+
+static int fsp_opal_set_sai(uint64_t async_token, char *loc_code,
+			    const u64 led_mask, const u64 led_value)
+{
+	int state = LED_STATE_OFF;
+
+	if (!((led_mask >> OPAL_SLOT_LED_TYPE_ATTN) & OPAL_SLOT_LED_STATE_ON))
+		return OPAL_PARAMETER;
+
+	if ((led_value >> OPAL_SLOT_LED_TYPE_ATTN) & OPAL_SLOT_LED_STATE_ON)
+		state = LED_STATE_ON;
+
+	return queue_led_state_change(loc_code, 0,
+				      state, SPCN_SRC_OPAL, async_token);
+}
+
 /*
  * fsp_opal_leds_get_ind (OPAL_LEDS_GET_INDICATOR)
  *
@@ -1116,6 +1282,7 @@ static int64_t fsp_opal_leds_get_ind(char *loc_code, u64 *led_mask,
 {
 	bool supported = true;
 	int64_t max;
+	int rc;
 	struct fsp_led_data *led;
 
 	/* FSP not present */
@@ -1137,6 +1304,12 @@ static int64_t fsp_opal_leds_get_ind(char *loc_code, u64 *led_mask,
 	if (max <= 0)
 		return OPAL_PARAMETER;
 
+	/* Get System attention indicator state */
+	if (is_sai_loc_code(loc_code)) {
+		rc = fsp_opal_get_sai(led_mask, led_value);
+		return rc;
+	}
+
 	/* LED not found */
 	led = fsp_find_cec_led(loc_code);
 	if (!led)
@@ -1222,6 +1395,14 @@ static int64_t fsp_opal_leds_set_ind(uint64_t async_token,
 	if (max <= 0)
 		return OPAL_PARAMETER;
 
+	/* Set System attention indicator state */
+	if (is_sai_loc_code(loc_code)) {
+		supported = true;
+		rc = fsp_opal_set_sai(async_token,
+				      loc_code, led_mask, led_value);
+		goto success;
+	}
+
 	/* LED not found */
 	led = fsp_find_cec_led(loc_code);
 	if (!led)
@@ -1274,6 +1455,50 @@ success:
 	return rc;
 }
 
+/* Get LED node from device tree */
+static struct dt_node *dt_get_led_node(void)
+{
+	struct dt_node *pled;
+
+	if (!opal_node) {
+		prlog(PR_WARNING, "OPAL parent device node not available\n");
+		return NULL;
+	}
+
+	pled = dt_find_by_path(opal_node, DT_PROPERTY_LED_NODE);
+	if (!pled)
+		prlog(PR_WARNING, "Parent device node not available\n");
+
+	return pled;
+}
+
+/* Get System attention indicator location code from device tree */
+static void dt_get_sai_loc_code(void)
+{
+	struct dt_node *pled, *child;
+	const char *led_type = NULL;
+
+	memset(sai_data.loc_code, 0, LOC_CODE_SIZE);
+
+	pled = dt_get_led_node();
+	if (!pled)
+		return;
+
+	list_for_each(&pled->children, child, list) {
+		led_type = dt_prop_get(child, DT_PROPERTY_LED_TYPES);
+		if (!led_type)
+			continue;
+
+		if (strcmp(led_type, LED_TYPE_ATTENTION))
+			continue;
+
+		memcpy(sai_data.loc_code, child->name, LOC_CODE_SIZE - 1);
+
+		prlog(PR_TRACE, "SAI Location code = %s\n", sai_data.loc_code);
+		return;
+	}
+}
+
 /*
  * create_led_device_node
  *
@@ -1285,6 +1510,7 @@ success:
  */
 void create_led_device_nodes(void)
 {
+	const char *led_mode = NULL;
 	struct fsp_led_data *led, *next;
 	struct dt_node *pled, *cled;
 
@@ -1296,47 +1522,47 @@ void create_led_device_nodes(void)
 		opal_run_pollers();
 
 	if (led_support == LED_STATE_ABSENT) {
-		prlog(PR_WARNING, PREFIX "LED support not available, \
+		prlog(PR_WARNING, "LED support not available, \
 		      hence device tree nodes will not be created\n");
 		return;
 	}
 
-	if (!opal_node) {
-		prlog(PR_WARNING, PREFIX
-		      "OPAL parent device node not available\n");
+	/* Get LED node */
+	pled = dt_get_led_node();
+	if (!pled)
 		return;
-	}
 
-	/* LED parent node */
-	pled = dt_new(opal_node, "led");
-	if (!pled) {
-		prlog(PR_WARNING, PREFIX
-		      "Parent device node creation failed\n");
+	dt_add_property_strings(pled, "compatible", DT_PROPERTY_LED_COMPATIBLE);
+
+	led_mode = dt_prop_get(pled, DT_PROPERTY_LED_MODE);
+	if (!led_mode) {
+		prlog(PR_WARNING, "Unknown LED operating mode\n");
 		return;
 	}
-	dt_add_property_strings(pled, "compatible", "ibm,opal-v3-led");
 
 	/* LED child nodes */
 	list_for_each_safe(&cec_ledq, led, next, link) {
 		/* Duplicate LED location code */
 		if (dt_find_by_path(pled, led->loc_code)) {
-			prlog(PR_WARNING, PREFIX "duplicate location code %s",
+			prlog(PR_WARNING, "duplicate location code %s",
 			      led->loc_code);
 			continue;
 		}
 
 		cled = dt_new(pled, led->loc_code);
 		if (!cled) {
-			prlog(PR_WARNING, PREFIX
-			      "Child device node creation failed\n");
+			prlog(PR_WARNING, "Child device node creation "
+			      "failed\n");
 			continue;
 		}
 
-		dt_add_property_strings(cled, "led-types", "identify", "fault");
-		if (is_enclosure_led(led->loc_code))
-			dt_add_property_strings(cled, "led-loc", "enclosure");
+		if (!strcmp(led_mode, LED_MODE_LIGHT_PATH))
+			dt_add_property_strings(cled, DT_PROPERTY_LED_TYPES,
+						LED_TYPE_IDENTIFY,
+						LED_TYPE_FAULT);
 		else
-			dt_add_property_strings(cled, "led-loc", "descendent");
+			dt_add_property_strings(cled, DT_PROPERTY_LED_TYPES,
+						LED_TYPE_IDENTIFY);
 	}
 }
 
@@ -1443,9 +1669,8 @@ static void replay_spcn_cmd(u32 last_spcn_cmd)
 					     PSI_DMA_LED_BUF),
 				   fsp_read_leds_data_complete);
 		if (rc)
-			prlog(PR_ERR, PREFIX
-			       "Replay SPCN_MOD_PRS_LED_DATA_FIRST"
-			       " command could not be queued\n");
+			prlog(PR_ERR, "Replay SPCN_MOD_PRS_LED_DATA_FIRST"
+			      " command could not be queued\n");
 	}
 
 	if (last_spcn_cmd == SPCN_MOD_PRS_LED_DATA_SUB) {
@@ -1455,9 +1680,8 @@ static void replay_spcn_cmd(u32 last_spcn_cmd)
 					     0, PSI_DMA_LED_BUF),
 				   fsp_read_leds_data_complete);
 		if (rc)
-			prlog(PR_ERR, PREFIX
-			       "Replay SPCN_MOD_PRS_LED_DATA_SUB"
-			       " command could not be queued\n");
+			prlog(PR_ERR, "Replay SPCN_MOD_PRS_LED_DATA_SUB"
+			      " command could not be queued\n");
 	}
 
 	/* Failed to queue MBOX message */
@@ -1490,8 +1714,8 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 
 	if (msg_status != FSP_STATUS_SUCCESS) {
 		log_simple_error(&e_info(OPAL_RC_LED_SUPPORT),
-			PREFIX "FSP returned error %x LED not supported\n",
-								 msg_status);
+				 "FSP returned error %x LED not supported\n",
+				 msg_status);
 		/* LED support not available */
 		led_support = LED_STATE_ABSENT;
 
@@ -1503,8 +1727,7 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 	switch (led_status) {
 	/* Last 1KB of LED data */
 	case SPCN_RSP_STATUS_SUCCESS:
-		prlog(PR_DEBUG, PREFIX
-		      "SPCN_RSP_STATUS_SUCCESS: %d bytes received\n",
+		prlog(PR_DEBUG, "SPCN_RSP_STATUS_SUCCESS: %d bytes received\n",
 		      data_len);
 
 		led_support = LED_STATE_PRESENT;
@@ -1513,10 +1736,9 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 		fsp_process_leds_data(data_len);
 
 		/* LEDs captured on the system */
-		prlog(PR_DEBUG, PREFIX
-		      "CEC LEDs captured on the system:\n");
+		prlog(PR_DEBUG, "CEC LEDs captured on the system:\n");
 		list_for_each_safe(&cec_ledq, led, next, link) {
-			prlog(PR_DEBUG, PREFIX
+			prlog(PR_DEBUG,
 			       "rid: %x\t"
 			       "len: %x      "
 			       "lcode: %-30s\t"
@@ -1529,9 +1751,9 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 			       led->status);
 		}
 
-		prlog(PR_DEBUG, PREFIX "ENCL LEDs captured on the system:\n");
+		prlog(PR_DEBUG, "ENCL LEDs captured on the system:\n");
 		list_for_each_safe(&encl_ledq, led, next, link) {
-			prlog(PR_DEBUG, PREFIX
+			prlog(PR_DEBUG,
 			       "rid: %x\t"
 			       "len: %x      "
 			       "lcode: %-30s\t"
@@ -1548,8 +1770,7 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 
 	/* If more 1KB of LED data present */
 	case SPCN_RSP_STATUS_COND_SUCCESS:
-		prlog(PR_DEBUG, PREFIX
-		      "SPCN_RSP_STATUS_COND_SUCCESS: %d bytes "
+		prlog(PR_DEBUG, "SPCN_RSP_STATUS_COND_SUCCESS: %d bytes "
 		      " received\n", data_len);
 
 		/* Copy data to the local list */
@@ -1563,8 +1784,8 @@ static void fsp_read_leds_data_complete(struct fsp_msg *msg)
 					     cmd_hdr, 0, PSI_DMA_LED_BUF),
 				   fsp_read_leds_data_complete);
 		if (rc) {
-			prlog(PR_ERR, PREFIX "SPCN_MOD_PRS_LED_DATA_SUB command"
-			       " could not be queued\n");
+			prlog(PR_ERR, "SPCN_MOD_PRS_LED_DATA_SUB command"
+			      " could not be queued\n");
 
 			led_support = LED_STATE_ABSENT;
 		}
@@ -1633,9 +1854,9 @@ static void fsp_leds_query_spcn(void)
 			SPCN_ADDR_MODE_CEC_NODE, cmd_hdr, 0,
 				PSI_DMA_LED_BUF), fsp_read_leds_data_complete);
 	if (rc)
-		prlog(PR_ERR, PREFIX
-		       "SPCN_MOD_PRS_LED_DATA_FIRST command could"
-		       " not be queued\n");
+		prlog(PR_ERR,
+		      "SPCN_MOD_PRS_LED_DATA_FIRST command could"
+		      " not be queued\n");
 	else	/* Initiated LED list fetch MBOX command */
 		led_support = LED_STATE_READING;
 }
@@ -1654,13 +1875,20 @@ void fsp_led_init(void)
 	list_head_init(&spcn_cmdq);
 
 	fsp_leds_query_spcn();
-	prlog(PR_TRACE, PREFIX "Init completed\n");
+	prlog(PR_TRACE, "Init completed\n");
+
+	/* Get System attention indicator state */
+	dt_get_sai_loc_code();
+	fsp_get_sai();
 
 	/* Handle FSP initiated async LED commands */
 	fsp_register_client(&fsp_indicator_client, FSP_MCLASS_INDICATOR);
-	prlog(PR_TRACE, PREFIX "FSP async command client registered\n");
+	prlog(PR_TRACE, "FSP async command client registered\n");
+
+	/* Register for SAI update notification */
+	sysparam_add_update_notifier(sai_update_notification);
 
 	opal_register(OPAL_LEDS_GET_INDICATOR, fsp_opal_leds_get_ind, 4);
 	opal_register(OPAL_LEDS_SET_INDICATOR, fsp_opal_leds_set_ind, 5);
-	prlog(PR_TRACE, PREFIX "LED OPAL interface registered\n");
+	prlog(PR_TRACE, "LED OPAL interface registered\n");
 }
