@@ -40,7 +40,7 @@
 #include <ccan/list/list.h>
 
 DEFINE_LOG_ENTRY(OPAL_RC_FSP_POLL_TIMEOUT, OPAL_PLATFORM_ERR_EVT, OPAL_FSP,
-		 OPAL_PLATFORM_FIRMWARE, OPAL_ERROR_PANIC, OPAL_NA, NULL);
+		 OPAL_PLATFORM_FIRMWARE, OPAL_ERROR_PANIC, OPAL_NA);
 
 #define FSP_TRACE_MSG
 #define FSP_TRACE_EVENT
@@ -1799,6 +1799,8 @@ static void fsp_init_links(struct dt_node *fsp_node)
 	struct fsp_iopath *fiop;
 
 	linksprop = dt_find_property(fsp_node, "ibm,psi-links");
+	assert(linksprop);
+
 	index = dt_prop_get_u32(fsp_node, "reg");
 	fsp = fsp_find_by_index(index);
 	if (!fsp) {
@@ -2263,6 +2265,7 @@ static void fsp_fetch_lid_complete(struct fsp_msg *msg)
 	if (rc !=0 && rc != 2) {
 		last->result = -EIO;
 		last = list_pop(&fsp_fetch_lid_queue, struct fsp_fetch_lid_item, link);
+		prerror("FSP LID %08x load ERROR %d\n", last->lid_no, rc);
 		list_add_tail(&fsp_fetched_lid, &last->link);
 		fsp_start_fetching_next_lid();
 		unlock(&fsp_fetch_lock);
@@ -2332,9 +2335,9 @@ static void fsp_fetch_lid_next_chunk(struct fsp_fetch_lid_item *last)
 	last->bsize = ((boff + chunk) + TCE_MASK) & ~TCE_MASK;
 	last->chunk_requested = chunk;
 
-	prlog(PR_DEBUG, "FSP: Loading Chunk 0x%08x bytes balign=%llx"
+	prlog(PR_DEBUG, "FSP: LID %08x chunk 0x%08x bytes balign=%llx"
 	      " boff=%llx bsize=%llx\n",
-	      chunk, balign, boff, last->bsize);
+	      last->lid_no, chunk, balign, boff, last->bsize);
 
 	fsp_tce_map(PSI_DMA_FETCH, (void *)balign, last->bsize);
 	taddr = PSI_DMA_FETCH + boff;
@@ -2353,6 +2356,7 @@ static void fsp_fetch_lid_next_chunk(struct fsp_fetch_lid_item *last)
 				struct fsp_fetch_lid_item, link);
 		list_add_tail(&fsp_fetched_lid, &last->link);
 	}
+	last->result = OPAL_BUSY;
 }
 
 static void fsp_start_fetching_next_lid(void)
@@ -2366,7 +2370,9 @@ static void fsp_start_fetching_next_lid(void)
 	if (last == NULL)
 		return;
 
-	fsp_fetch_lid_next_chunk(last);
+	/* If we're not already fetching */
+	if (last->result == OPAL_EMPTY)
+		fsp_fetch_lid_next_chunk(last);
 }
 
 int fsp_start_preload_resource(enum resource_id id, uint32_t idx,
@@ -2387,7 +2393,7 @@ int fsp_start_preload_resource(enum resource_id id, uint32_t idx,
 	resource->remaining = *size;
 	*size = 0;
 	resource->length = size;
-	resource->result = OPAL_BUSY;
+	resource->result = OPAL_EMPTY;
 
 	for (i = 0; i < ARRAY_SIZE(fsp_lid_map); i++) {
 		if (id != fsp_lid_map[i].id)
@@ -2429,10 +2435,8 @@ int fsp_resource_loaded(enum resource_id id, uint32_t idx)
 
 	if (resource) {
 		rc = resource->result;
-		if (rc == OPAL_SUCCESS) {
-			list_del(&resource->link);
-			free(resource);
-		}
+		list_del(&resource->link);
+		free(resource);
 	}
 	unlock(&fsp_fetch_lock);
 
@@ -2465,10 +2469,10 @@ static int fsp_lid_loaded(uint32_t lid_no)
 	return rc;
 }
 
-int fsp_load_lid(uint32_t lid_no, char *buf, size_t *size)
+int fsp_preload_lid(uint32_t lid_no, char *buf, size_t *size)
 {
 	struct fsp_fetch_lid_item *resource;
-	int r;
+	int r = OPAL_SUCCESS;
 
 	resource = malloc(sizeof(struct fsp_fetch_lid_item));
 	assert(resource != NULL);
@@ -2481,7 +2485,7 @@ int fsp_load_lid(uint32_t lid_no, char *buf, size_t *size)
 	resource->remaining = *size;
 	*size = 0;
 	resource->length = size;
-	resource->result = OPAL_BUSY;
+	resource->result = OPAL_EMPTY;
 
 	if (lid_no == 0)
 		return OPAL_PARAMETER;
@@ -2495,17 +2499,27 @@ int fsp_load_lid(uint32_t lid_no, char *buf, size_t *size)
 	fsp_start_fetching_next_lid();
 	unlock(&fsp_fetch_lock);
 
+	return r;
+}
+
+int fsp_wait_lid_loaded(uint32_t lid_no)
+{
+	int r;
+	int waited = 0;
+
 	r = fsp_lid_loaded(lid_no);
 
 	while(r == OPAL_BUSY) {
 		opal_run_pollers();
 		time_wait_nopoll(msecs_to_tb(5));
+		waited+=5;
 		cpu_relax();
 		r = fsp_lid_loaded(lid_no);
 	}
 
-	return r;
+	prlog(PR_DEBUG, "FSP: fsp_wait_lid_loaded %x %u ms\n", lid_no, waited);
 
+	return r;
 }
 
 void fsp_used_by_console(void)
