@@ -46,7 +46,7 @@ static struct lock psi_lock = LOCK_UNLOCKED;
 
 DEFINE_LOG_ENTRY(OPAL_RC_PSI_TIMEOUT, OPAL_PLATFORM_ERR_EVT, OPAL_PSI,
 		OPAL_PLATFORM_FIRMWARE,
-		OPAL_UNRECOVERABLE_ERR_LOSS_OF_FUNCTION, OPAL_NA, NULL);
+		OPAL_UNRECOVERABLE_ERR_LOSS_OF_FUNCTION, OPAL_NA);
 
 void psi_set_link_polling(bool active)
 {
@@ -885,6 +885,8 @@ static bool psi_init_psihb(struct dt_node *psihb)
 	psi->chip_id = chip->id;
 	psi->interrupt = get_psi_interrupt(chip->id);
 
+	chip->psi = psi;
+
 	psi_create_mm_dtnode(psi);
 	psi_register_interrupts(psi);
 	psi_activate_phb(psi);
@@ -905,6 +907,11 @@ void psi_fsp_link_in_use(struct psi *psi __unused)
 	}
 }
 
+struct psi *psi_find_functional_chip(void)
+{
+	return list_top(&psis, struct psi, list);
+}
+
 void psi_init(void)
 {
 	struct dt_node *np;
