@@ -338,6 +338,8 @@
 #define FSP_RSP_ERRLOG_PHYP_ACK	0x0ce8800 /* FSP->HV */
 #define FSP_CMD_ERRLOG_GET_PLID	0x0ce0900 /* FSP->HV: Get PLID */
 #define FSP_RSP_ERRLOG_GET_PLID	0x0ce8900 /* HV->FSP */
+#define FSP_CMD_SA_INDICATOR	0x1ce1000 /* HV->FSP: read/update SAI */
+#define FSP_RSP_SA_INDICATOR	0x0ce9000 /* FSP->HV */
 #define FSP_CMD_QUERY_SPARM	0x1ce1200 /* HV->FSP: System parameter query */
 #define FSP_RSP_QUERY_SPARM	0x0ce9200 /* FSP->HV: System parameter resp */
 #define FSP_CMD_SET_SPARM_1	0x1ce1301 /* HV->FSP: Set system parameter */
@@ -370,6 +372,16 @@
 #define FSP_CMD_PANELSTATUS_EX1	0x0ce5c02 /* FSP->HV */
 #define FSP_CMD_PANELSTATUS_EX2	0x0ce5c03 /* FSP->HV */
 
+/* SAI read/update sub commands */
+#define FSP_LED_RESET_REAL_SAI		0x00
+#define FSP_LED_READ_REAL_SAI		0x02
+#define FSP_LED_RESET_PARTITION_SAI	0x80
+#define FSP_LED_SET_PARTITION_SAI	0x81
+#define FSP_LED_READ_PARTITION_SAI	0x82
+#define FSP_LED_READ_PLAT_SAI		0x83
+#define FSP_LED_RESET_PLAT_SAI		0x84
+#define FSP_LED_SET_PLAT_SAI		0x85
+
 /*
  * Class 0xD2
  */
@@ -409,6 +421,9 @@
  */
 #define FSP_CMD_FETCH_SP_DATA	0x1d40101 /* HV->FSP: Fetch & DMA data */
 #define FSP_CMD_WRITE_SP_DATA	0x1d40201 /* HV->FSP: Fetch & DMA data */
+#define FSP_CMD_FETCH_PLAT_DATA	0x1d40500 /* HV->FSP: Platform function data */
+#define FSP_CMD_SEND_PLAT_DATA	0x0d40501 /* FSP->HV */
+#define FSP_RSP_PLAT_DATA	0x0d48500 /* HV->FSP */
 
 /* Data set IDs for SP data commands */
 #define FSP_DATASET_SP_DUMP	0x01
@@ -507,6 +522,12 @@
 #define FSP_CMD_VSERIAL_IN	0x0e10100 /* FSP->HV */
 #define FSP_CMD_VSERIAL_OUT	0x0e10200 /* HV->FSP */
 
+/*
+ * Class E6
+ */
+#define FSP_CMD_TOPO_ENABLE_DISABLE	0x0e60600 /* FSP->HV */
+#define FSP_RSP_TOPO_ENABLE_DISABLE	0x0e68600 /* HV->FSP */
+
 /*
  * Class E8
  */
@@ -725,7 +746,8 @@ extern int fsp_fetch_data_queue(uint8_t flags, uint16_t id, uint32_t sub_id,
 extern int fsp_start_preload_resource(enum resource_id id, uint32_t idx,
 				      void *buf, size_t *size);
 extern int fsp_resource_loaded(enum resource_id id, uint32_t idx);
-extern int fsp_load_lid(uint32_t lid_no, char *buf, size_t *size);
+extern int fsp_preload_lid(uint32_t lid_no, char *buf, size_t *size);
+extern int fsp_wait_lid_loaded(uint32_t lid_no);
 
 /* FSP console stuff */
 extern void fsp_console_preinit(void);
@@ -774,6 +796,9 @@ extern int (*fsp_flash_term_hook)(void);
 extern void fsp_init_surveillance(void);
 extern void fsp_surv_query(void);
 
+/* IPMI */
+extern void fsp_ipmi_init(void);
+
 /* Reset/Reload */
 extern void fsp_reinit_fsp(void);
 extern void fsp_trigger_reset(void);
@@ -799,4 +824,7 @@ extern void fsp_epow_init(void);
 /* DPO */
 extern void fsp_dpo_init(void);
 
+/* Chiptod */
+extern void fsp_chiptod_init(void);
+
 #endif /* __FSP_H */
