@@ -40,10 +40,11 @@ DECLARE_PLATFORM(firestone) = {
 	.name			= "Firestone",
 	.probe			= firestone_probe,
 	.init			= astbmc_init,
-	.external_irq		= astbmc_ext_irq,
+	.external_irq		= astbmc_ext_irq_serirq_cpld,
 	.cec_power_down         = astbmc_ipmi_power_down,
 	.cec_reboot             = astbmc_ipmi_reboot,
 	.elog_commit		= ipmi_elog_commit,
-	.start_preload_resource	= flash_load_resource,
+	.start_preload_resource	= flash_start_preload_resource,
+	.resource_loaded	= flash_resource_loaded,
 	.exit			= ipmi_wdt_final_reset,
 };
